@@ -29,6 +29,7 @@ static bool should_add_braces(chunk_t *vbopen);
 
 void do_braces(void)
 {
+   LOG_FUNC_ENTRY();
    if (cpd.settings[UO_mod_full_brace_if_chain].b)
    {
       mod_full_brace_if_chain();
@@ -111,6 +112,7 @@ void do_braces(void)
  */
 static void examine_braces(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *prev;
 
@@ -151,6 +153,7 @@ static void examine_braces(void)
  */
 static bool should_add_braces(chunk_t *vbopen)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    int     nl_max   = cpd.settings[UO_mod_full_brace_nl].n;
    int     nl_count = 0;
@@ -186,6 +189,7 @@ static bool should_add_braces(chunk_t *vbopen)
  */
 static bool can_remove_braces(chunk_t *bopen)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *prev      = NULL;
    int     semi_count = 0;
@@ -326,6 +330,7 @@ static bool can_remove_braces(chunk_t *bopen)
  */
 static void examine_brace(chunk_t *bopen)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *next;
    chunk_t *prev      = NULL;
@@ -411,6 +416,7 @@ static void examine_brace(chunk_t *bopen)
                 (pc->type == CT_FOR) ||
                 (pc->type == CT_DO) ||
                 (pc->type == CT_WHILE) ||
+                (pc->type == CT_SWITCH) ||
                 (pc->type == CT_USING_STMT) ||
                 ((pc->type == CT_BRACE_OPEN) && was_fcn))
             {
@@ -498,6 +504,7 @@ static void examine_brace(chunk_t *bopen)
  */
 static void convert_brace(chunk_t *br)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *tmp;
 
    if (!br || (br->flags & PCF_KEEP_BRACE))
@@ -506,13 +513,13 @@ static void convert_brace(chunk_t *br)
    }
    else if (br->type == CT_BRACE_OPEN)
    {
-      br->type = CT_VBRACE_OPEN;
+      set_chunk_type(br, CT_VBRACE_OPEN);
       br->str.clear();
       tmp = chunk_get_prev(br);
    }
    else if (br->type == CT_BRACE_CLOSE)
    {
-      br->type = CT_VBRACE_CLOSE;
+      set_chunk_type(br, CT_VBRACE_CLOSE);
       br->str.clear();
       tmp = chunk_get_next(br);
    }
@@ -543,13 +550,14 @@ static void convert_brace(chunk_t *br)
  */
 static void convert_vbrace(chunk_t *vbr)
 {
+   LOG_FUNC_ENTRY();
    if (vbr == NULL)
    {
       return;
    }
    else if (vbr->type == CT_VBRACE_OPEN)
    {
-      vbr->type = CT_BRACE_OPEN;
+      set_chunk_type(vbr, CT_BRACE_OPEN);
       vbr->str  = "{";
 
       /* If the next chunk is a preprocessor, then move the open brace after the
@@ -565,7 +573,7 @@ static void convert_vbrace(chunk_t *vbr)
    }
    else if (vbr->type == CT_VBRACE_CLOSE)
    {
-      vbr->type = CT_BRACE_CLOSE;
+      set_chunk_type(vbr, CT_BRACE_CLOSE);
       vbr->str  = "}";
 
       /* If the next chunk is a comment, followed by a newline, then
@@ -588,6 +596,7 @@ static void convert_vbrace(chunk_t *vbr)
 
 static void convert_vbrace_to_brace(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *tmp;
    chunk_t *vbc;
@@ -662,6 +671,7 @@ static void convert_vbrace_to_brace(void)
 chunk_t *insert_comment_after(chunk_t *ref, c_token_t cmt_type,
                               const unc_text& cmt_text)
 {
+   LOG_FUNC_ENTRY();
    chunk_t new_cmt;
 
    new_cmt      = *ref;
@@ -692,6 +702,47 @@ chunk_t *insert_comment_after(chunk_t *ref, c_token_t cmt_type,
 }
 
 
+/**
+ * Collect the text into txt that contains the full tag name.
+ * Mainly for collecting namespace 'a.b.c' or function 'foo::bar()' names.
+ */
+static void append_tag_name(unc_text& txt, chunk_t *pc)
+{
+   LOG_FUNC_ENTRY();
+   chunk_t *tmp = pc;
+
+   /* step backwards over all a::b stuff */
+   while ((tmp = chunk_get_prev_ncnl(tmp)) != NULL)
+   {
+      if ((tmp->type != CT_DC_MEMBER) && (tmp->type != CT_MEMBER))
+      {
+         break;
+      }
+      tmp = chunk_get_prev_ncnl(tmp);
+      pc = tmp;
+      if (!chunk_is_word(tmp))
+      {
+         break;
+      }
+   }
+
+   txt += pc->str;
+   while ((pc = chunk_get_next_ncnl(pc)) != NULL)
+   {
+      if ((pc->type != CT_DC_MEMBER) && (pc->type != CT_MEMBER))
+      {
+         break;
+      }
+      txt += pc->str;
+      pc = chunk_get_next_ncnl(pc);
+      if (pc)
+      {
+         txt += pc->str;
+      }
+   }
+}
+
+
 /*
  * See also it's preprocessor counterpart
  *   add_long_preprocessor_conditional_block_comment
@@ -699,16 +750,18 @@ chunk_t *insert_comment_after(chunk_t *ref, c_token_t cmt_type,
  */
 void add_long_closebrace_comment(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *tmp;
    chunk_t *br_open;
    chunk_t *br_close;
    chunk_t *fcn_pc = NULL;
    chunk_t *sw_pc  = NULL;
+   chunk_t *ns_pc  = NULL;
+   unc_text xstr;
    int     nl_count;
 
-   pc = chunk_get_head();
-   while ((pc = chunk_get_next_ncnl(pc)) != NULL)
+   for (pc = chunk_get_head(); pc; pc = chunk_get_next_ncnl(pc))
    {
       if ((pc->type == CT_FUNC_DEF) ||
           (pc->type == CT_OC_MSG_DECL))
@@ -720,6 +773,10 @@ void add_long_closebrace_comment(void)
          /* kinda pointless, since it always has the text "switch" */
          sw_pc = pc;
       }
+      else if (pc->type == CT_NAMESPACE)
+      {
+         ns_pc = pc;
+      }
       if ((pc->type != CT_BRACE_OPEN) || ((pc->flags & PCF_IN_PREPROC) != 0))
       {
          continue;
@@ -754,12 +811,25 @@ void add_long_closebrace_comment(void)
                {
                   nl_min = cpd.settings[UO_mod_add_long_switch_closebrace_comment].n;
                   tag_pc = sw_pc;
+                  xstr   = sw_pc->str;
                }
                else if ((br_open->parent_type == CT_FUNC_DEF) ||
                         (br_open->parent_type == CT_OC_MSG_DECL))
                {
                   nl_min = cpd.settings[UO_mod_add_long_function_closebrace_comment].n;
                   tag_pc = fcn_pc;
+                  xstr.clear();
+                  append_tag_name(xstr, tag_pc);
+               }
+               else if (br_open->parent_type == CT_NAMESPACE)
+               {
+                  nl_min = cpd.settings[UO_mod_add_long_namespace_closebrace_comment].n;
+                  tag_pc = ns_pc;
+                  /* obtain the next chunck, normally this is the name of the namespace
+                     and append it to generate "namespace xyz" */
+                  xstr = ns_pc->str;
+                  xstr.append(" ");
+                  append_tag_name(xstr, chunk_get_next(ns_pc));
                }
 
                if ((nl_min > 0) && (nl_count >= nl_min) && (tag_pc != NULL))
@@ -769,7 +839,7 @@ void add_long_closebrace_comment(void)
                                     CT_COMMENT_CPP : CT_COMMENT;
 
                   /* Add a comment after the close brace */
-                  insert_comment_after(br_close, style, tag_pc->str);
+                  insert_comment_after(br_close, style, xstr);
                }
             }
             break;
@@ -781,6 +851,7 @@ void add_long_closebrace_comment(void)
 
 static void move_case_break(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *prev = NULL;
 
@@ -807,6 +878,7 @@ static void move_case_break(void)
  */
 static chunk_t *mod_case_brace_remove(chunk_t *br_open)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *br_close;
    chunk_t *next = chunk_get_next_ncnl(br_open, CNAV_PREPROC);
@@ -864,6 +936,7 @@ static chunk_t *mod_case_brace_remove(chunk_t *br_open)
  */
 static chunk_t *mod_case_brace_add(chunk_t *cl_colon)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc   = cl_colon;
    chunk_t *last = NULL;
    chunk_t *next = chunk_get_next_ncnl(cl_colon, CNAV_PREPROC);
@@ -934,6 +1007,7 @@ static chunk_t *mod_case_brace_add(chunk_t *cl_colon)
 
 static void mod_case_brace(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc = chunk_get_head();
    chunk_t *next;
 
@@ -972,6 +1046,7 @@ static void mod_case_brace(void)
  */
 static void process_if_chain(chunk_t *br_start)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *braces[256];
    chunk_t *br_close;
    int     br_cnt = 0;
@@ -1083,6 +1158,7 @@ static void process_if_chain(chunk_t *br_start)
 
 static void mod_full_brace_if_chain(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
 
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
