@@ -51,6 +51,7 @@ chunk_t *chunk_get_next_nc(chunk_t *cur, chunk_nav_t nav = CNAV_ALL);
 chunk_t *chunk_get_next_nnl(chunk_t *cur, chunk_nav_t nav = CNAV_ALL);
 chunk_t *chunk_get_next_ncnl(chunk_t *cur, chunk_nav_t nav = CNAV_ALL);
 chunk_t *chunk_get_next_ncnlnp(chunk_t *cur, chunk_nav_t nav = CNAV_ALL);
+chunk_t *chunk_get_next_nisq(chunk_t *cur, chunk_nav_t nav = CNAV_ALL);
 
 chunk_t *chunk_get_next_nblank(chunk_t *cur, chunk_nav_t nav = CNAV_ALL);
 chunk_t *chunk_get_prev_nblank(chunk_t *cur, chunk_nav_t nav = CNAV_ALL);
@@ -296,5 +297,33 @@ bool chunk_safe_to_del_nl(chunk_t *nl)
    return(chunk_same_preproc(chunk_get_prev(nl), chunk_get_next(nl)));
 }
 
+/**
+ * Handle for (... in ...) in Objective-C.
+ * Returns true if pc->prev points to a CT_FOR and we find a CT_IN before the closing parenthesis.
+ */
+static_inline
+bool chunk_is_forin(chunk_t *pc)
+{
+   if ((cpd.lang_flags & LANG_OC) && pc && (pc->type == CT_SPAREN_OPEN))
+   {
+      chunk_t *prev = chunk_get_prev_ncnl(pc);
+      if (prev->type == CT_FOR)
+      {
+         chunk_t *next = pc;
+         while (next && (next->type != CT_SPAREN_CLOSE) && (next->type != CT_IN))
+         {
+            next = chunk_get_next_ncnl(next);
+         }
+         if (next->type == CT_IN)
+         {
+            return true;
+         }
+      }
+   }
+   return false;
+}
 
-#endif   /* CHUNK_LIST_H_INCLUDED */
+void set_chunk_type(chunk_t *pc, c_token_t tt);
+void set_chunk_parent(chunk_t *pc, c_token_t tt);
+
+#endif /* CHUNK_LIST_H_INCLUDED */
