@@ -5,10 +5,9 @@
  * @author  Ben Gardner
  * @license GPL v2+
  */
-#ifdef WIN32
+#if defined(_WIN32) && !defined(__CYGWIN__)
 
 #include "windows_compat.h"
-#include "windows.h"
 #include <string>
 #include <cstdio>
 
@@ -58,4 +57,4 @@ bool unc_homedir(std::string& home)
    return false;
 }
 
-#endif /* ifdef WIN32 */
+#endif /* if defined(_WIN32) && !defined(__CYGWIN__) */
