@@ -117,10 +117,12 @@
  */
 
 static void indent_comment(chunk_t *pc, int col);
+static bool ifdef_over_whole_file();
 
 
 void indent_to_column(chunk_t *pc, int column)
 {
+   LOG_FUNC_ENTRY();
    if (column < pc->column)
    {
       column = pc->column;
@@ -139,6 +141,7 @@ enum align_mode
 /* Same as indent_to_column, except we can move both ways */
 void align_to_column(chunk_t *pc, int column)
 {
+   LOG_FUNC_ENTRY();
    if (column == pc->column)
    {
       return;
@@ -217,11 +220,14 @@ void align_to_column(chunk_t *pc, int column)
  * @param pc      The chunk at the start of the line
  * @param column  The desired column
  */
-void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno)
+void reindent_line(chunk_t *pc, int column)
 {
-   LOG_FMT(LINDLINE, "%s: %d] col %d on %s [%s] => %d <called from '%s' line %d\n",
+   LOG_FUNC_ENTRY();
+   LOG_FMT(LINDLINE, "%s: %d] col %d on '%s' [%s/%s] => %d",
            __func__, pc->orig_line, pc->column, pc->str.c_str(),
-           get_token_name(pc->type), column, fcn_name, lineno);
+           get_token_name(pc->type), get_token_name(pc->parent_type),
+           column);
+   log_func_stack_inline(LINDLINE);
 
    if (column == pc->column)
    {
@@ -240,6 +246,11 @@ void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno)
       {
          break;
       }
+      if (pc->nl_count)
+      {
+         min_col   = 0;
+         col_delta = 0;
+      }
       min_col += space_col_align(pc, next);
       pc       = next;
 
@@ -279,6 +290,7 @@ void reindent_line2(chunk_t *pc, int column, const char *fcn_name, int lineno)
  */
 static void indent_pse_push(struct parse_frame& frm, chunk_t *pc)
 {
+   LOG_FUNC_ENTRY();
    static int ref = 0;
 
    /* check the stack depth */
@@ -300,6 +312,7 @@ static void indent_pse_push(struct parse_frame& frm, chunk_t *pc)
       frm.pse[frm.pse_tos].indent_tab  = frm.pse[frm.pse_tos - 1].indent_tab;
       frm.pse[frm.pse_tos].indent_cont = frm.pse[frm.pse_tos - 1].indent_cont;
       frm.pse[frm.pse_tos].non_vardef  = false;
+      frm.pse[frm.pse_tos].ns_cnt      = frm.pse[frm.pse_tos - 1].ns_cnt;
       memcpy(&frm.pse[frm.pse_tos].ip, &frm.pse[frm.pse_tos - 1].ip, sizeof(frm.pse[frm.pse_tos].ip));
    }
 }
@@ -313,6 +326,7 @@ static void indent_pse_push(struct parse_frame& frm, chunk_t *pc)
  */
 static void indent_pse_pop(struct parse_frame& frm, chunk_t *pc)
 {
+   LOG_FUNC_ENTRY();
    /* Bump up the index and initialize it */
    if (frm.pse_tos > 0)
    {
@@ -393,10 +407,18 @@ static int calc_indent_continue(struct parse_frame& frm, int pse_tos)
  * We are on a '{' that has parent = OC_BLOCK_EXPR
  * find the column of the param tag
  */
-static chunk_t *oc_msg_block_indent(chunk_t *pc)
+static chunk_t *oc_msg_block_indent(chunk_t *pc, bool from_brace,
+                                    bool from_caret, bool from_colon,
+                                    bool from_keyword)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *tmp = chunk_get_prev_nc(pc);
 
+   if (from_brace)
+   {
+      return pc;
+   }
+
    if (chunk_is_paren_close(tmp))
    {
       tmp = chunk_get_prev_nc(chunk_skip_to_match_rev(tmp));
@@ -405,17 +427,38 @@ static chunk_t *oc_msg_block_indent(chunk_t *pc)
    {
       return NULL;
    }
+   if (from_caret)
+   {
+      return tmp;
+   }
    tmp = chunk_get_prev_nc(tmp);
    if (!tmp || (tmp->type != CT_OC_COLON))
    {
       return NULL;
    }
+   if (from_colon)
+   {
+      return tmp;
+   }
    tmp = chunk_get_prev_nc(tmp);
    if (!tmp || ((tmp->type != CT_OC_MSG_NAME) && (tmp->type != CT_OC_MSG_FUNC)))
    {
       return NULL;
    }
-   return tmp;
+   if (from_keyword)
+   {
+      return tmp;
+   }
+   return NULL;
+}
+
+
+/**
+ * We are on a '{' that has parent = OC_BLOCK_EXPR
+ */
+static chunk_t *oc_msg_prev_colon(chunk_t *pc)
+{
+   return chunk_get_prev_type(pc, CT_OC_COLON, pc->level, CNAV_ALL);
 }
 
 
@@ -426,6 +469,7 @@ static chunk_t *oc_msg_block_indent(chunk_t *pc)
  */
 void indent_text(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t            *pc;
    chunk_t            *next;
    chunk_t            *prev       = NULL;
@@ -531,9 +575,9 @@ void indent_text(void)
          {
             next = chunk_get_next(pc);
             /* Hack to get the logs to look right */
-            next->type = CT_PP_REGION_INDENT;
+            set_chunk_type(next, CT_PP_REGION_INDENT);
             indent_pse_push(frm, next);
-            next->type = CT_PP_REGION;
+            set_chunk_type(next, CT_PP_REGION);
 
             /* Indent one level */
             frm.pse[frm.pse_tos].indent     = frm.pse[frm.pse_tos - 1].indent + indent_size;
@@ -550,13 +594,14 @@ void indent_text(void)
             next = chunk_get_next(pc);
             /* Hack to get the logs to look right */
             memtype = next->type;
-            next->type = CT_PP_IF_INDENT;
+            set_chunk_type(next, CT_PP_IF_INDENT);
             indent_pse_push(frm, next);
-            next->type = memtype;
+            set_chunk_type(next, memtype);
 
-            /* Indent one level */
-            frm.pse[frm.pse_tos].indent     = frm.pse[frm.pse_tos - 1].indent + indent_size;
-            frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos - 1].indent_tab + indent_size;
+            /* Indent one level except if the #if is a #include guard */
+            int extra = ((pc->pp_level == 0) && ifdef_over_whole_file()) ? 0 : indent_size;
+            frm.pse[frm.pse_tos].indent     = frm.pse[frm.pse_tos - 1].indent + extra;
+            frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos - 1].indent_tab + extra;
             frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
             frm.pse[frm.pse_tos].in_preproc = false;
          }
@@ -673,6 +718,7 @@ void indent_text(void)
                  (pc->type == CT_COMMA) ||
                  (pc->type == CT_BRACE_OPEN) ||
                  (pc->type == CT_SPAREN_CLOSE) ||
+                 ((pc->type == CT_SQUARE_OPEN) && (pc->parent_type == CT_OC_AT)) ||
                  ((pc->type == CT_SQUARE_OPEN) && (pc->parent_type == CT_ASSIGN))) &&
                  (pc->parent_type != CT_CPP_LAMBDA))
             {
@@ -697,7 +743,8 @@ void indent_text(void)
             }
 
             /* End any CPP/ObjC class colon stuff */
-            if ((frm.pse[frm.pse_tos].type == CT_CLASS_COLON) &&
+            if (((frm.pse[frm.pse_tos].type == CT_CLASS_COLON) ||
+                 (frm.pse[frm.pse_tos].type == CT_CONSTR_COLON)) &&
                 ((pc->type == CT_BRACE_OPEN) ||
                  (pc->type == CT_OC_END) ||
                  (pc->type == CT_OC_SCOPE) ||
@@ -757,6 +804,15 @@ void indent_text(void)
                indent_pse_pop(frm, pc);
             }
 
+            /* an CLASS is ended with a semicolon or brace open */
+            if ((frm.pse[frm.pse_tos].type == CT_CLASS) &&
+                ((pc->type == CT_CLASS_COLON) ||
+                 (pc->type == CT_BRACE_OPEN) ||
+                 chunk_is_semicolon(pc)))
+            {
+               indent_pse_pop(frm, pc);
+            }
+
             /* Close out parens and squares */
             if ((frm.pse[frm.pse_tos].type == (pc->type - 1)) &&
                 ((pc->type == CT_PAREN_CLOSE) ||
@@ -774,14 +830,26 @@ void indent_text(void)
       /* Grab a copy of the current indent */
       indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
 
-      if (!chunk_is_newline(pc) && !chunk_is_comment(pc))
+      if (!chunk_is_newline(pc) && !chunk_is_comment(pc) && log_sev_on(LINDPC))
       {
-         LOG_FMT(LINDPC, " -=[ %s ]=- top=%d %s %d/%d\n",
-                 pc->str.c_str(),
-                 frm.pse_tos,
-                 get_token_name(frm.pse[frm.pse_tos].type),
-                 frm.pse[frm.pse_tos].indent_tmp,
-                 frm.pse[frm.pse_tos].indent);
+         LOG_FMT(LINDPC, " -=[ %d:%d %s ]=-\n",
+                 pc->orig_line, pc->orig_col, pc->str.c_str());
+         for (int ttidx = frm.pse_tos; ttidx > 0; ttidx--)
+         {
+            LOG_FMT(LINDPC, "     [%d %d:%d %s/%s tmp=%d ind=%d bri=%d tab=%d cont=%d lvl=%d blvl=%d]\n",
+                    ttidx,
+                    frm.pse[ttidx].pc->orig_line,
+                    frm.pse[ttidx].pc->orig_col,
+                    get_token_name(frm.pse[ttidx].type),
+                    get_token_name(frm.pse[ttidx].pc->parent_type),
+                    frm.pse[ttidx].indent_tmp,
+                    frm.pse[ttidx].indent,
+                    frm.pse[ttidx].brace_indent,
+                    frm.pse[ttidx].indent_tab,
+                    frm.pse[ttidx].indent_cont,
+                    frm.pse[ttidx].level,
+                    frm.pse[ttidx].pc->brace_level);
+         }
       }
 
       /**
@@ -807,7 +875,7 @@ void indent_text(void)
                          (!cpd.settings[UO_indent_braces_no_func].b ||
                           (pc->parent_type != CT_FUNC_DEF)) &&
                          (!cpd.settings[UO_indent_braces_no_func].b ||
-                          (pc->parent_type != CT_FUNC_CLASS)) &&
+                          (pc->parent_type != CT_FUNC_CLASS_DEF)) &&
                          (!cpd.settings[UO_indent_braces_no_class].b ||
                           (pc->parent_type != CT_CLASS)) &&
                          (!cpd.settings[UO_indent_braces_no_struct].b ||
@@ -848,21 +916,80 @@ void indent_text(void)
          frm.level++;
          indent_pse_push(frm, pc);
 
-         if (frm.paren_count != 0)
+         /* any '{' that is inside of a '(' overrides the '(' indent */
+         if (!cpd.settings[UO_indent_paren_open_brace].b &&
+             chunk_is_paren_open(frm.pse[frm.pse_tos - 1].pc) &&
+             chunk_is_newline(chunk_get_next_nc(pc)))
+         {
+            /* FIXME: I don't know how much of this is necessary, but it seems to work */
+            frm.pse[frm.pse_tos].brace_indent = 1 + (pc->brace_level * indent_size);
+            indent_column = frm.pse[frm.pse_tos].brace_indent;
+            frm.pse[frm.pse_tos].indent       = indent_column + indent_size;
+            frm.pse[frm.pse_tos].indent_tab   = frm.pse[frm.pse_tos].indent;
+            frm.pse[frm.pse_tos].indent_tmp   = frm.pse[frm.pse_tos].indent;
+
+            frm.pse[frm.pse_tos - 1].indent_tmp = frm.pse[frm.pse_tos].indent_tmp;
+         }
+         else if (frm.paren_count != 0)
          {
             if (frm.pse[frm.pse_tos].pc->parent_type == CT_OC_BLOCK_EXPR)
             {
                if ((pc->flags & PCF_IN_OC_MSG) &&
                    cpd.settings[UO_indent_oc_block_msg].n)
                {
-                  frm.pse[frm.pse_tos].ip.ref   = oc_msg_block_indent(pc);
+                  frm.pse[frm.pse_tos].ip.ref   = oc_msg_block_indent(pc, false, false, false, true);
                   frm.pse[frm.pse_tos].ip.delta = cpd.settings[UO_indent_oc_block_msg].n;
                }
 
-               if (cpd.settings[UO_indent_oc_block].b)
+               if (cpd.settings[UO_indent_oc_block].b ||
+                   cpd.settings[UO_indent_oc_block_msg_xcode_style].b)
                {
-                  frm.pse[frm.pse_tos].indent = 1 + (pc->level * indent_size);
-                  indent_column_set(frm.pse[frm.pse_tos].indent - indent_size);
+                  bool in_oc_msg = (pc->flags & PCF_IN_OC_MSG);
+                  bool indent_from_keyword = cpd.settings[UO_indent_oc_block_msg_from_keyword].b && in_oc_msg;
+                  bool indent_from_colon = cpd.settings[UO_indent_oc_block_msg_from_colon].b && in_oc_msg;
+                  bool indent_from_caret = cpd.settings[UO_indent_oc_block_msg_from_caret].b && in_oc_msg;
+                  bool indent_from_brace = cpd.settings[UO_indent_oc_block_msg_from_brace].b && in_oc_msg;
+
+                  // In "Xcode indent mode", we want to indent:
+                  //  - if the colon is aligned (namely, if a newline has been
+                  //    added before it), indent_from_brace
+                  //  - otherwise, indent from previous block (the "else" statement here)
+                  if (cpd.settings[UO_indent_oc_block_msg_xcode_style].b)
+                  {
+                     chunk_t *colon = oc_msg_prev_colon(pc);
+                     chunk_t *param_name = chunk_get_prev(colon);
+                     chunk_t *before_param = chunk_get_prev(param_name);
+
+                     if (before_param && (before_param->type == CT_NEWLINE))
+                     {
+                        indent_from_keyword = true;
+                        indent_from_colon   = false;
+                        indent_from_caret   = false;
+                        indent_from_brace   = false;
+                     }
+                     else
+                     {
+                        indent_from_brace   = false;
+                        indent_from_colon   = false;
+                        indent_from_caret   = false;
+                        indent_from_keyword = false;
+                     }
+                  }
+
+                  chunk_t *ref = oc_msg_block_indent(pc, indent_from_brace,
+                                                     indent_from_caret,
+                                                     indent_from_colon,
+                                                     indent_from_keyword);
+                  if (ref)
+                  {
+                     frm.pse[frm.pse_tos].indent = indent_size + ref->column;
+                     indent_column_set(frm.pse[frm.pse_tos].indent - indent_size);
+                  }
+                  else
+                  {
+                     frm.pse[frm.pse_tos].indent = 1 + ((pc->brace_level + 1) * indent_size);
+                     indent_column_set(frm.pse[frm.pse_tos].indent - indent_size);
+                  }
                }
                else
                {
@@ -922,8 +1049,18 @@ void indent_text(void)
             }
             else if (pc->parent_type == CT_NAMESPACE)
             {
-               if ((pc->flags & PCF_LONG_BLOCK) ||
-                   !cpd.settings[UO_indent_namespace].b)
+               if (cpd.settings[UO_indent_namespace].b &&
+                   cpd.settings[UO_indent_namespace_single_indent].b)
+               {
+                  if (frm.pse[frm.pse_tos].ns_cnt)
+                  {
+                     /* undo indent on all except the first namespace */
+                     frm.pse[frm.pse_tos].indent -= indent_size;
+                  }
+                  indent_column_set((frm.pse_tos <= 1) ? 1 : frm.pse[frm.pse_tos - 1].brace_indent);
+               }
+               else if ((pc->flags & PCF_LONG_BLOCK) ||
+                        !cpd.settings[UO_indent_namespace].b)
                {
                   /* don't indent long blocks */
                   frm.pse[frm.pse_tos].indent -= indent_size;
@@ -937,6 +1074,7 @@ void indent_text(void)
                         cpd.settings[UO_indent_namespace_level].n;
                   }
                }
+               frm.pse[frm.pse_tos].ns_cnt++;
             }
             else if ((pc->parent_type == CT_EXTERN) && !cpd.settings[UO_indent_extern].b)
             {
@@ -1099,7 +1237,16 @@ void indent_text(void)
             }
          }
       }
-      else if (pc->type == CT_CLASS_COLON)
+      else if (pc->type == CT_CLASS)
+      {
+         frm.level++;
+         indent_pse_push(frm, pc);
+         frm.pse[frm.pse_tos].indent     = frm.pse[frm.pse_tos - 1].indent + indent_size;
+         frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
+         frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos].indent;
+      }
+      else if ((pc->type == CT_CLASS_COLON) ||
+               (pc->type == CT_CONSTR_COLON))
       {
          /* just indent one level */
          indent_pse_push(frm, pc);
@@ -1109,7 +1256,8 @@ void indent_text(void)
 
          indent_column_set(frm.pse[frm.pse_tos].indent_tmp);
 
-         if (cpd.settings[UO_indent_class_colon].b)
+         if ((cpd.settings[UO_indent_class_colon].b && (pc->type == CT_CLASS_COLON)) ||
+             (cpd.settings[UO_indent_constr_colon].b && (pc->type == CT_CONSTR_COLON)))
          {
             prev = chunk_get_prev(pc);
             if (chunk_is_newline(prev))
@@ -1146,8 +1294,20 @@ void indent_text(void)
          bool skipped = false;
 
          indent_pse_push(frm, pc);
+         if (chunk_is_newline(chunk_get_prev(pc)) &&
+             (pc->column != indent_column))
+         {
+            LOG_FMT(LINDENT, "%s: %d] indent => %d [%s]\n",
+                    __func__, pc->orig_line, indent_column, pc->str.c_str());
+            reindent_line(pc, indent_column);
+         }
          frm.pse[frm.pse_tos].indent = pc->column + pc->len();
 
+         if ((pc->type == CT_SQUARE_OPEN) && (cpd.lang_flags & LANG_D))
+         {
+            frm.pse[frm.pse_tos].indent_tab = frm.pse[frm.pse_tos].indent;
+         }
+
          if (((pc->type == CT_FPAREN_OPEN) || (pc->type == CT_ANGLE_OPEN)) &&
              ((cpd.settings[UO_indent_func_call_param].b &&
                ((pc->parent_type == CT_FUNC_CALL) ||
@@ -1155,10 +1315,11 @@ void indent_text(void)
               ||
               (cpd.settings[UO_indent_func_proto_param].b &&
                ((pc->parent_type == CT_FUNC_PROTO) ||
-                (pc->parent_type == CT_FUNC_CLASS)))
+                (pc->parent_type == CT_FUNC_CLASS_PROTO)))
               ||
               (cpd.settings[UO_indent_func_class_param].b &&
-               (pc->parent_type == CT_FUNC_CLASS))
+               ((pc->parent_type == CT_FUNC_CLASS_DEF) ||
+                (pc->parent_type == CT_FUNC_CLASS_PROTO)))
               ||
               (cpd.settings[UO_indent_template_param].b &&
                (pc->parent_type == CT_TEMPLATE))
@@ -1180,6 +1341,7 @@ void indent_text(void)
                    (frm.pse[idx].type != CT_SQUARE_OPEN) &&
                    (frm.pse[idx].type != CT_ANGLE_OPEN) &&
                    (frm.pse[idx].type != CT_CLASS_COLON) &&
+                   (frm.pse[idx].type != CT_CONSTR_COLON) &&
                    (frm.pse[idx].type != CT_ASSIGN_NL))
             {
                idx--;
@@ -1233,8 +1395,7 @@ void indent_text(void)
          {
             frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent;
             if ((pc->level == pc->brace_level) &&
-                (((pc->type == CT_FPAREN_OPEN) && (pc->parent_type != CT_FUNC_CTOR_VAR)) ||
-                 (pc->type == CT_SPAREN_OPEN)))
+                ((pc->type == CT_FPAREN_OPEN) || (pc->type == CT_SPAREN_OPEN)))
             {
                //frm.pse[frm.pse_tos].indent += abs(cpd.settings[UO_indent_continue].n);
                frm.pse[frm.pse_tos].indent = calc_indent_continue(frm, frm.pse_tos);
@@ -1355,6 +1516,13 @@ void indent_text(void)
          else
          {
             vardefcol = pc->column;
+            /* need to skip backward over any '*' */
+            chunk_t *tmp = chunk_get_prev_nc(pc);
+            while (chunk_is_token(tmp, CT_PTR_TYPE))
+            {
+               vardefcol = tmp->column;
+               tmp = chunk_get_prev_nc(tmp);
+            }
          }
       }
       if (chunk_is_semicolon(pc) ||
@@ -1387,6 +1555,28 @@ void indent_text(void)
 
          prev = chunk_get_prev_ncnl(pc);
          next = chunk_get_next_ncnl(pc);
+
+         bool do_vardefcol = false;
+         if ((vardefcol > 0) &&
+             (pc->level == pc->brace_level) &&
+             prev &&
+             ((prev->type == CT_COMMA) ||
+              (prev->type == CT_TYPE) ||
+              (prev->type == CT_PTR_TYPE) ||
+              (prev->type == CT_WORD)))
+         {
+            chunk_t *tmp = pc;
+            while (chunk_is_token(tmp, CT_PTR_TYPE))
+            {
+               tmp = chunk_get_next_ncnl(tmp);
+            }
+            if (tmp && ((tmp->flags & PCF_VAR_DEF) != 0) &&
+                ((tmp->type == CT_WORD) || (tmp->type == CT_FUNC_CTOR_VAR)))
+            {
+               do_vardefcol = true;
+            }
+         }
+
          if ((pc->flags & PCF_DONT_INDENT) != 0)
          {
             /* no change */
@@ -1410,19 +1600,21 @@ void indent_text(void)
                     __func__, pc->orig_line, tmp);
             reindent_line(pc, tmp);
          }
-         else if ((vardefcol > 0) &&
-                  (pc->level == pc->brace_level) &&
-                  ((pc->type == CT_WORD) || (pc->type == CT_FUNC_CTOR_VAR)) &&
-                  (prev != NULL) &&
-                  ((prev->type == CT_COMMA) ||
-                   (prev->type == CT_TYPE) ||
-                   (prev->type == CT_WORD)) &&
-                  ((pc->flags & PCF_VAR_DEF) != 0))
+         else if (do_vardefcol)
          {
             LOG_FMT(LINDENT, "%s: %d] Vardefcol => %d\n",
                     __func__, pc->orig_line, vardefcol);
             reindent_line(pc, vardefcol);
          }
+         else if ((pc->type == CT_NAMESPACE) &&
+                  cpd.settings[UO_indent_namespace].b &&
+                  cpd.settings[UO_indent_namespace_single_indent].b &&
+                  frm.pse[frm.pse_tos].ns_cnt)
+         {
+            LOG_FMT(LINDENT, "%s: %d] Namespace => %d\n",
+                    __func__, pc->orig_line, frm.pse[frm.pse_tos].brace_indent);
+            reindent_line(pc, frm.pse[frm.pse_tos].brace_indent);
+         }
          else if ((pc->type == CT_STRING) && (prev != NULL) && (prev->type == CT_STRING) &&
                   cpd.settings[UO_indent_align_string].b)
          {
@@ -1631,6 +1823,12 @@ void indent_text(void)
       indent_pse_pop(frm, pc);
    }
 
+   /* Throw out any VBRACE_OPEN at the end - implied with the end of file  */
+   while ((frm.pse_tos > 0) && (frm.pse[frm.pse_tos].type == CT_VBRACE_OPEN))
+   {
+      indent_pse_pop(frm, pc);
+   }
+
    for (idx = 1; idx <= frm.pse_tos; idx++)
    {
       LOG_FMT(LWARN, "%s:%d Unmatched %s\n",
@@ -1651,6 +1849,7 @@ void indent_text(void)
  */
 static bool single_line_comment_indent_rule_applies(chunk_t *start)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc      = start;
    int     nl_count = 0;
 
@@ -1725,6 +1924,7 @@ static bool single_line_comment_indent_rule_applies(chunk_t *start)
  */
 static void indent_comment(chunk_t *pc, int col)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *nl;
    chunk_t *prev;
 
@@ -1786,11 +1986,18 @@ static void indent_comment(chunk_t *pc, int col)
  */
 static bool ifdef_over_whole_file()
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *next;
 
    int stage = 0;
 
+   /* the results for this file are cached */
+   if (cpd.ifdef_over_whole_file)
+   {
+      return (cpd.ifdef_over_whole_file > 0);
+   }
+
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
    {
       if (chunk_is_comment(pc) || chunk_is_newline(pc))
@@ -1834,12 +2041,10 @@ static bool ifdef_over_whole_file()
       }
    }
 
-   if (stage == 2)
-   {
-      LOG_FMT(LINFO, "The whole file is covered by a #IF\n");
-      return(true);
-   }
-   return(stage == 2);
+   cpd.ifdef_over_whole_file = (stage == 2) ? 1 : -1;
+   LOG_FMT(LNOTE, "The whole file is%s covered by a #IF\n",
+           (cpd.ifdef_over_whole_file > 0) ? "" : " NOT");
+   return (cpd.ifdef_over_whole_file > 0);
 }
 
 
@@ -1850,6 +2055,7 @@ static bool ifdef_over_whole_file()
  */
 void indent_preproc(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *next;
    int     pp_level;
