@@ -31,9 +31,10 @@ static const chunk_tag_t keywords[] =
    { "@catch",           CT_CATCH,        LANG_OC | LANG_CPP | LANG_C                                                 },
    { "@dynamic",         CT_OC_DYNAMIC,   LANG_OC | LANG_CPP | LANG_C                                                 },
    { "@end",             CT_OC_END,       LANG_OC | LANG_CPP | LANG_C                                                 },
-   { "@finally",         CT_TRY,          LANG_OC | LANG_CPP | LANG_C                                                 },
+   { "@finally",         CT_FINALLY,      LANG_OC | LANG_CPP | LANG_C                                                 },
    { "@implementation",  CT_OC_IMPL,      LANG_OC | LANG_CPP | LANG_C                                                 },
    { "@interface",       CT_OC_INTF,      LANG_OC | LANG_CPP | LANG_C                                                 },
+   { "@interface",       CT_CLASS,        LANG_JAVA                                                                   },
    { "@private",         CT_PRIVATE,      LANG_OC | LANG_CPP | LANG_C                                                 },
    { "@property",        CT_OC_PROPERTY,  LANG_OC | LANG_CPP | LANG_C                                                 },
    { "@protocol",        CT_OC_PROTOCOL,  LANG_OC | LANG_CPP | LANG_C                                                 },
@@ -41,6 +42,8 @@ static const chunk_tag_t keywords[] =
    { "@synthesize",      CT_OC_DYNAMIC,   LANG_OC | LANG_CPP | LANG_C                                                 },
    { "@throw",           CT_THROW,        LANG_OC                                                                     },
    { "@try",             CT_TRY,          LANG_OC | LANG_CPP | LANG_C                                                 },
+   { "NS_ENUM",          CT_ENUM,         LANG_OC                                                                     },
+   { "NS_OPTIONS",       CT_ENUM,         LANG_OC                                                                     },
    { "_Bool",            CT_TYPE,         LANG_CPP                                                                    },
    { "_Complex",         CT_TYPE,         LANG_CPP                                                                    },
    { "_Imaginary",       CT_TYPE,         LANG_CPP                                                                    },
@@ -100,6 +103,7 @@ static const chunk_tag_t keywords[] =
    { "dchar",            CT_TYPE,         LANG_D                                                                      },
    { "debug",            CT_DEBUG,        LANG_D                                                                      },
    { "debugger",         CT_DEBUGGER,     LANG_ECMA                                                                   },
+   { "decltype",         CT_SIZEOF,       LANG_CPP                                                                    },
    { "default",          CT_DEFAULT,      LANG_ALL                                                                    }, // PAWN
    { "define",           CT_PP_DEFINE,    LANG_ALL | FLAG_PP                                                          }, // PAWN
    { "defined",          CT_DEFINED,      LANG_PAWN                                                                   }, // PAWN
@@ -129,8 +133,8 @@ static const chunk_tag_t keywords[] =
    { "extern",           CT_EXTERN,       LANG_C | LANG_CPP | LANG_CS | LANG_D | LANG_VALA                            },
    { "false",            CT_WORD,         LANG_CPP | LANG_CS | LANG_D | LANG_JAVA | LANG_VALA                         },
    { "file",             CT_PP_FILE,      LANG_PAWN | FLAG_PP                                                         }, // PAWN
-   { "final",            CT_QUALIFIER,    LANG_D | LANG_ECMA                                                          },
-   { "finally",          CT_FINALLY,      LANG_D | LANG_CS | LANG_ECMA                                                },
+   { "final",            CT_QUALIFIER,    LANG_CPP | LANG_D | LANG_ECMA                                               },
+   { "finally",          CT_FINALLY,      LANG_D | LANG_CS | LANG_ECMA | LANG_JAVA                                    },
    { "flags",            CT_TYPE,         LANG_VALA                                                                   },
    { "float",            CT_TYPE,         LANG_ALLC                                                                   },
    { "for",              CT_FOR,          LANG_ALL                                                                    }, // PAWN
@@ -151,7 +155,7 @@ static const chunk_tag_t keywords[] =
    { "implicit",         CT_QUALIFIER,    LANG_CS                                                                     },
    { "import",           CT_IMPORT,       LANG_D | LANG_JAVA | LANG_ECMA                                              }, // fudged to get indenting
    { "import",           CT_PP_INCLUDE,   LANG_OC | FLAG_PP                                                           }, // ObjectiveC version of include
-   { "in",               CT_IN,           LANG_D | LANG_CS | LANG_VALA | LANG_ECMA                                    },
+   { "in",               CT_IN,           LANG_D | LANG_CS | LANG_VALA | LANG_ECMA | LANG_OC                          },
    { "include",          CT_PP_INCLUDE,   LANG_C | LANG_CPP | LANG_PAWN | FLAG_PP                                     }, // PAWN
    { "inline",           CT_QUALIFIER,    LANG_C | LANG_CPP                                                           },
    { "inout",            CT_QUALIFIER,    LANG_D                                                                      },
@@ -170,7 +174,7 @@ static const chunk_tag_t keywords[] =
    { "mixin",            CT_CLASS,        LANG_D                                                                      }, // may need special handling
    { "module",           CT_D_MODULE,     LANG_D                                                                      },
    { "mutable",          CT_QUALIFIER,    LANG_C | LANG_CPP                                                           },
-   { "namespace",        CT_NAMESPACE,    LANG_C | LANG_CPP | LANG_CS | LANG_VALA                                     },
+   { "namespace",        CT_NAMESPACE,    LANG_CPP | LANG_CS | LANG_VALA                                              },
    { "native",           CT_NATIVE,       LANG_PAWN                                                                   }, // PAWN
    { "native",           CT_QUALIFIER,    LANG_JAVA | LANG_ECMA                                                       },
    { "new",              CT_NEW,          LANG_CPP | LANG_CS | LANG_D | LANG_JAVA | LANG_PAWN | LANG_VALA | LANG_ECMA }, // PAWN
@@ -188,6 +192,7 @@ static const chunk_tag_t keywords[] =
    { "params",           CT_TYPE,         LANG_CS                                                                     },
    { "pragma",           CT_PP_PRAGMA,    LANG_ALL | FLAG_PP                                                          }, // PAWN
    { "private",          CT_PRIVATE,      LANG_ALLC                                                                   }, // not C
+   { "property",         CT_PP_PROPERTY,  LANG_CS | FLAG_PP                                                           },
    { "protected",        CT_PRIVATE,      LANG_ALLC                                                                   }, // not C
    { "public",           CT_PRIVATE,      LANG_ALL                                                                    }, // PAWN // not C
    { "readonly",         CT_QUALIFIER,    LANG_CS                                                                     },
