@@ -100,6 +100,10 @@ enum log_sev_t
    LFLPAREN  = 86,    /* Flag paren */
    LOCMSGD   = 87,    /* OC Message declaration */
    LINDENTAG = 88,    /* indent again */
+   LNFD      = 89,    /* newline-function-def */
+   LJDBI     = 90,    /* Java Double Brace Init */
+   LSETPAR   = 91,    /* set_chunk_parent() */
+   LSETTYP   = 92,    /* set_chunk_type() */
 };
 
 #endif   /* LOG_LEVELS_H_INCLUDED */
