@@ -11,6 +11,7 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
+#include <algorithm>
 #include <cerrno>
 #include "unc_ctype.h"
 
@@ -19,11 +20,13 @@ static void newlines_double_space_struct_enum_union(chunk_t *open_brace);
 static bool one_liner_nl_ok(chunk_t *pc);
 static void undo_one_liner(chunk_t *pc);
 static void nl_handle_define(chunk_t *pc);
+
 static void newline_iarf_pair(chunk_t *before, chunk_t *after, argval_t av);
 
 #define MARK_CHANGE()    mark_change(__func__, __LINE__)
 static void mark_change(const char *func, int line)
 {
+   LOG_FUNC_ENTRY();
    cpd.changes++;
    if (cpd.pass_count == 0)
    {
@@ -41,6 +44,7 @@ static void mark_change(const char *func, int line)
  */
 static bool can_increase_nl(chunk_t *nl)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *prev = chunk_get_prev_nc(nl);
    chunk_t *pcmt = chunk_get_prev(nl);
    chunk_t *next = chunk_get_next(nl);
@@ -100,6 +104,7 @@ static bool can_increase_nl(chunk_t *nl)
  */
 static void double_newline(chunk_t *nl)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *prev = chunk_get_prev(nl);
 
    LOG_FMT(LNEWLINE, "%s: add newline after %s on line %d",
@@ -133,6 +138,7 @@ static void double_newline(chunk_t *nl)
 //#define DEBUG_NEWLINES
 static void setup_newline_add(chunk_t *prev, chunk_t *nl, chunk_t *next)
 {
+   LOG_FUNC_ENTRY();
    if (!prev || !nl || !next)
    {
       return;
@@ -152,12 +158,12 @@ static void setup_newline_add(chunk_t *prev, chunk_t *nl, chunk_t *next)
    }
    if ((nl->flags & PCF_IN_PREPROC) != 0)
    {
-      nl->type = CT_NL_CONT;
+      set_chunk_type(nl, CT_NL_CONT);
       nl->str  = "\\\n";
    }
    else
    {
-      nl->type = CT_NEWLINE;
+      set_chunk_type(nl, CT_NEWLINE);
       nl->str  = "\n";
    }
 }
@@ -167,8 +173,9 @@ static void setup_newline_add(chunk_t *prev, chunk_t *nl, chunk_t *next)
  * Add a newline before the chunk if there isn't already a newline present.
  * Virtual braces are skipped, as they do not contribute to the output.
  */
-chunk_t *newline_add_before2(chunk_t *pc, const char *fcn, int line)
+chunk_t *newline_add_before(chunk_t *pc)
 {
+   LOG_FUNC_ENTRY();
    chunk_t nl;
    chunk_t *prev;
 
@@ -179,8 +186,9 @@ chunk_t *newline_add_before2(chunk_t *pc, const char *fcn, int line)
       return(prev);
    }
 
-   LOG_FMT(LNEWLINE, "%s: '%s' on line %d (called from %s line %d)\n",
-           __func__, pc->str.c_str(), pc->orig_line, fcn, line);
+   LOG_FMT(LNEWLINE, "%s: '%s' on line %d",
+           __func__, pc->str.c_str(), pc->orig_line);
+   log_func_stack_inline(LSETTYP);
 
    setup_newline_add(prev, &nl, pc);
 
@@ -189,9 +197,10 @@ chunk_t *newline_add_before2(chunk_t *pc, const char *fcn, int line)
 }
 
 
-chunk_t *newline_force_before2(chunk_t *pc, const char *fcn, int line)
+chunk_t *newline_force_before(chunk_t *pc)
 {
-   chunk_t *nl = newline_add_before2(pc, fcn, line);
+   LOG_FUNC_ENTRY();
+   chunk_t *nl = newline_add_before(pc);
    if (nl && (nl->nl_count > 1))
    {
       nl->nl_count = 1;
@@ -205,8 +214,9 @@ chunk_t *newline_force_before2(chunk_t *pc, const char *fcn, int line)
  * Add a newline after the chunk if there isn't already a newline present.
  * Virtual braces are skipped, as they do not contribute to the output.
  */
-chunk_t *newline_add_after2(chunk_t *pc, const char *fcn, int line)
+chunk_t *newline_add_after(chunk_t *pc)
 {
+   LOG_FUNC_ENTRY();
    chunk_t nl;
    chunk_t *next;
 
@@ -222,8 +232,9 @@ chunk_t *newline_add_after2(chunk_t *pc, const char *fcn, int line)
       return(next);
    }
 
-   LOG_FMT(LNEWLINE, "%s: '%s' on line %d (called from %s line %d)\n",
-           __func__, pc->str.c_str(), pc->orig_line, fcn, line);
+   LOG_FMT(LNEWLINE, "%s: '%s' on line %d",
+           __func__, pc->str.c_str(), pc->orig_line);
+   log_func_stack_inline(LNEWLINE);
 
    setup_newline_add(pc, &nl, next);
 
@@ -232,9 +243,10 @@ chunk_t *newline_add_after2(chunk_t *pc, const char *fcn, int line)
 }
 
 
-chunk_t *newline_force_after2(chunk_t *pc, const char *fcn, int line)
+chunk_t *newline_force_after(chunk_t *pc)
 {
-   chunk_t *nl = newline_add_after2(pc, fcn, line);
+   LOG_FUNC_ENTRY();
+   chunk_t *nl = newline_add_after(pc);
    if (nl && (nl->nl_count > 1))
    {
       nl->nl_count = 1;
@@ -249,6 +261,7 @@ chunk_t *newline_force_after2(chunk_t *pc, const char *fcn, int line)
  */
 static void newline_end_newline(chunk_t *br_close)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *next = chunk_get_next(br_close);
    chunk_t nl;
 
@@ -278,17 +291,15 @@ static void newline_end_newline(chunk_t *br_close)
 }
 
 
-#define newline_min_after(ref, cnt, flag) \
-   newline_min_after2(ref, cnt, flag, __func__, __LINE__)
-
-static void newline_min_after2(chunk_t *ref, INT32 count, UINT64 flag,
-                               const char *func, int line)
+static void newline_min_after(chunk_t *ref, INT32 count, UINT64 flag)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc = ref;
    chunk_t *next;
 
-   LOG_FMT(LNEWLINE, "%s: '%s' line %d - count=%d flg=0x%" PRIx64 ": caller=%s:%d\n",
-           __func__, ref->str.c_str(), ref->orig_line, count, flag, func, line);
+   LOG_FMT(LNEWLINE, "%s: '%s' line %d - count=%d flg=0x%" PRIx64 ":",
+           __func__, ref->str.c_str(), ref->orig_line, count, flag);
+   log_func_stack_inline(LNEWLINE);
 
    do
    {
@@ -335,9 +346,9 @@ static void newline_min_after2(chunk_t *ref, INT32 count, UINT64 flag,
  *    if (...)   //comment
  *    {
  */
-chunk_t *newline_add_between2(chunk_t *start, chunk_t *end,
-                              const char *func, int line)
+chunk_t *newline_add_between(chunk_t *start, chunk_t *end)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
 
    if ((start == NULL) || (end == NULL))
@@ -345,10 +356,11 @@ chunk_t *newline_add_between2(chunk_t *start, chunk_t *end,
       return(NULL);
    }
 
-   LOG_FMT(LNEWLINE, "%s: '%s'[%s] line %d:%d and '%s' line %d:%d : caller=%s:%d\n",
+   LOG_FMT(LNEWLINE, "%s: '%s'[%s] line %d:%d and '%s' line %d:%d :",
            __func__, start->str.c_str(), get_token_name(start->type),
            start->orig_line, start->orig_col,
-           end->str.c_str(), end->orig_line, end->orig_col, func, line);
+           end->str.c_str(), end->orig_line, end->orig_col);
+   log_func_stack_inline(LNEWLINE);
 
    /* Back-up check for one-liners (should never be true!) */
    if (!one_liner_nl_ok(start))
@@ -397,18 +409,19 @@ chunk_t *newline_add_between2(chunk_t *start, chunk_t *end,
  * @param end     The ending chunk (cannot be a newline)
  * @return        true/false - removed something
  */
-void newline_del_between2(chunk_t *start, chunk_t *end,
-                          const char *func, int line)
+void newline_del_between(chunk_t *start, chunk_t *end)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *next;
    chunk_t *prev;
    chunk_t *pc = start;
 
-   LOG_FMT(LNEWLINE, "%s: '%s' line %d:%d and '%s' line %d:%d : caller=%s:%d preproc=%d/%d\n",
+   LOG_FMT(LNEWLINE, "%s: '%s' line %d:%d and '%s' line %d:%d : preproc=%d/%d ",
            __func__, start->str.c_str(), start->orig_line, start->orig_col,
-           end->str.c_str(), end->orig_line, end->orig_col, func, line,
+           end->str.c_str(), end->orig_line, end->orig_col,
            ((start->flags & PCF_IN_PREPROC) != 0),
            ((end->flags & PCF_IN_PREPROC) != 0));
+   log_func_stack_inline(LNEWLINE);
 
    /* Can't remove anything if the preproc status differs */
    if (!chunk_same_preproc(start, end))
@@ -472,6 +485,7 @@ void newline_del_between2(chunk_t *start, chunk_t *end,
  */
 static bool newlines_if_for_while_switch(chunk_t *start, argval_t nl_opt)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *close_paren;
    chunk_t *brace_open;
@@ -547,6 +561,7 @@ static bool newlines_if_for_while_switch(chunk_t *start, argval_t nl_opt)
  */
 static void newlines_if_for_while_switch_pre_blank_lines(chunk_t *start, argval_t nl_opt)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *prev;
    chunk_t *next;
@@ -646,6 +661,7 @@ static void newlines_if_for_while_switch_pre_blank_lines(chunk_t *start, argval_
 
 static chunk_t *get_closing_brace(chunk_t *start)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    int     level = start->level;
 
@@ -672,6 +688,7 @@ static chunk_t *get_closing_brace(chunk_t *start)
  */
 static void remove_next_newlines(chunk_t *start)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *next;
 
    while ((next = chunk_get_next(start)) != NULL)
@@ -702,6 +719,7 @@ static void remove_next_newlines(chunk_t *start)
  */
 static void newlines_if_for_while_switch_post_blank_lines(chunk_t *start, argval_t nl_opt)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *next;
    chunk_t *prev;
@@ -863,6 +881,7 @@ static void newlines_if_for_while_switch_post_blank_lines(chunk_t *start, argval
  */
 static void newlines_struct_enum_union(chunk_t *start, argval_t nl_opt, bool leave_trailing)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *next;
 
@@ -921,6 +940,7 @@ static void newlines_struct_enum_union(chunk_t *start, argval_t nl_opt, bool lea
  */
 static void newlines_cuddle_uncuddle(chunk_t *start, argval_t nl_opt)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *br_close;
 
    if (((start->flags & PCF_IN_PREPROC) != 0) &&
@@ -943,6 +963,7 @@ static void newlines_cuddle_uncuddle(chunk_t *start, argval_t nl_opt)
  */
 static void newlines_do_else(chunk_t *start, argval_t nl_opt)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *next;
    chunk_t *tmp;
 
@@ -997,6 +1018,7 @@ static void newlines_do_else(chunk_t *start, argval_t nl_opt)
  */
 static chunk_t *newline_def_blk(chunk_t *start, bool fn_top)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *next;
    chunk_t *prev;
    chunk_t *pc;
@@ -1061,7 +1083,9 @@ static chunk_t *newline_def_blk(chunk_t *start, bool fn_top)
       }
 
       /* Determine if this is a variable def or code */
-      if (!did_this_line && (pc->type != CT_FUNC_CLASS) &&
+      if (!did_this_line &&
+          (pc->type != CT_FUNC_CLASS_DEF) &&
+          (pc->type != CT_FUNC_CLASS_PROTO) &&
           ((pc->level == (start->level + 1)) ||
            (pc->level == 0)))
       {
@@ -1191,6 +1215,7 @@ static chunk_t *newline_def_blk(chunk_t *start, bool fn_top)
  */
 static void newlines_brace_pair(chunk_t *br_open)
 {
+   LOG_FUNC_ENTRY();
    chunk_t  *prev;
    chunk_t  *next;
    chunk_t  *pc;
@@ -1264,9 +1289,10 @@ static void newlines_brace_pair(chunk_t *br_open)
    if ((br_open->parent_type == CT_FUNC_DEF) ||
        (br_open->parent_type == CT_FUNC_CALL) ||
        (br_open->parent_type == CT_FUNC_CALL_USER) ||
-       (br_open->parent_type == CT_FUNC_CLASS) ||
+       (br_open->parent_type == CT_FUNC_CLASS_DEF) ||
        (br_open->parent_type == CT_OC_MSG_DECL) ||
-       (br_open->parent_type == CT_CS_PROPERTY))
+       (br_open->parent_type == CT_CS_PROPERTY) ||
+       (br_open->parent_type == CT_CPP_LAMBDA))
    {
       /* Need to force a newline before the close brace, if not in a class body */
       if ((br_open->flags & PCF_IN_CLASS) == 0)
@@ -1279,12 +1305,14 @@ static void newlines_brace_pair(chunk_t *br_open)
       newline_add_between(br_open, pc);
 
       val = (((br_open->parent_type == CT_FUNC_DEF) ||
-              (br_open->parent_type == CT_FUNC_CLASS) ||
+              (br_open->parent_type == CT_FUNC_CLASS_DEF) ||
               (br_open->parent_type == CT_OC_MSG_DECL)) ?
              cpd.settings[UO_nl_fdef_brace].a :
              ((br_open->parent_type == CT_CS_PROPERTY) ?
               cpd.settings[UO_nl_property_brace].a :
-              cpd.settings[UO_nl_fcall_brace].a));
+              ((br_open->parent_type == CT_CPP_LAMBDA) ?
+               cpd.settings[UO_nl_cpp_ldef_brace].a :
+               cpd.settings[UO_nl_fcall_brace].a)));
 
       if (val != AV_IGNORE)
       {
@@ -1349,6 +1377,7 @@ static void newlines_brace_pair(chunk_t *br_open)
  */
 static void newline_case(chunk_t *start)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *prev = start;
 
@@ -1395,6 +1424,7 @@ static void newline_case(chunk_t *start)
 
 static void newline_case_colon(chunk_t *start)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc = start;
 
    /* Scan forwards until a non-comment is found */
@@ -1415,6 +1445,7 @@ static void newline_case_colon(chunk_t *start)
  */
 static void newline_before_return(chunk_t *start)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *nl;
    chunk_t *pc;
 
@@ -1461,6 +1492,7 @@ static void newline_before_return(chunk_t *start)
  */
 static void newline_after_return(chunk_t *start)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *semi;
    chunk_t *after;
@@ -1500,6 +1532,9 @@ static void newline_after_return(chunk_t *start)
  */
 static void newline_iarf_pair(chunk_t *before, chunk_t *after, argval_t av)
 {
+   LOG_FUNC_ENTRY();
+   log_func_stack(LNEWLINE, "Call Stack:");
+
    if ((before != NULL) && (after != NULL))
    {
       if ((av & AV_ADD) != 0)
@@ -1526,6 +1561,9 @@ static void newline_iarf_pair(chunk_t *before, chunk_t *after, argval_t av)
  */
 void newline_iarf(chunk_t *pc, argval_t av)
 {
+   LOG_FUNC_ENTRY();
+   log_func_stack(LNEWLINE, "CallStack:");
+
    newline_iarf_pair(pc, chunk_get_next_nnl(pc), av);
 }
 
@@ -1536,11 +1574,16 @@ void newline_iarf(chunk_t *pc, argval_t av)
  */
 static void newline_func_def(chunk_t *start)
 {
+   LOG_FUNC_ENTRY();
    chunk_t  *pc;
    chunk_t  *prev = NULL;
    chunk_t  *tmp;
    argval_t atmp;
-   bool     is_def = (start->parent_type == CT_FUNC_DEF) || (start->parent_type == CT_FUNC_CLASS);
+   bool     is_def = (start->parent_type == CT_FUNC_DEF) || (start->parent_type == CT_FUNC_CLASS_DEF);
+
+   LOG_FMT(LNFD, "%s: called on %d:%d '%s' [%s/%s]\n",
+           __func__, start->orig_line, start->orig_col,
+           start->text(), get_token_name(start->type), get_token_name(start->parent_type));
 
    atmp = cpd.settings[is_def ? UO_nl_func_def_paren : UO_nl_func_paren].a;
    if (atmp != AV_IGNORE)
@@ -1554,6 +1597,7 @@ static void newline_func_def(chunk_t *start)
 
    /* Handle break newlines type and function */
    prev = chunk_get_prev_ncnl(start);
+   prev = skip_template_prev(prev);
    /* Don't split up a function variable */
    prev = chunk_is_paren_close(prev) ? NULL : chunk_get_prev_ncnl(prev);
 
@@ -1576,39 +1620,47 @@ static void newline_func_def(chunk_t *start)
          }
       }
 
-      argval_t a = (tmp->parent_type == CT_FUNC_PROTO) ?
-                   cpd.settings[UO_nl_func_proto_type_name].a :
-                   cpd.settings[UO_nl_func_type_name].a;
-      if ((tmp->flags & PCF_IN_CLASS) &&
-          (cpd.settings[UO_nl_func_type_name_class].a != AV_IGNORE))
+      if (chunk_get_next_ncnl(prev)->type != CT_FUNC_CLASS_DEF)
       {
-         a = cpd.settings[UO_nl_func_type_name_class].a;
-      }
-
-      if (a != AV_IGNORE)
-      {
-         if ((prev != NULL) && (prev->type == CT_DESTRUCTOR))
+         argval_t a = (tmp->parent_type == CT_FUNC_PROTO) ?
+                      cpd.settings[UO_nl_func_proto_type_name].a :
+                      cpd.settings[UO_nl_func_type_name].a;
+         if ((tmp->flags & PCF_IN_CLASS) &&
+             (cpd.settings[UO_nl_func_type_name_class].a != AV_IGNORE))
          {
-            prev = chunk_get_prev_ncnl(prev);
+            a = cpd.settings[UO_nl_func_type_name_class].a;
          }
 
-         /* If we are on a '::', step back two tokens
-          * TODO: do we also need to check for '.' ?
-          */
-         while ((prev != NULL) && (prev->type == CT_DC_MEMBER))
+         if (a != AV_IGNORE)
          {
-            prev = chunk_get_prev_ncnl(prev);
-            prev = skip_template_prev(prev);
-            prev = chunk_get_prev_ncnl(prev);
-         }
+            LOG_FMT(LNFD, "%s: prev %d:%d '%s' [%s/%s]\n",
+                    __func__, prev->orig_line, prev->orig_col,
+                    prev->text(), get_token_name(prev->type), get_token_name(prev->parent_type));
 
-         if ((prev != NULL) &&
-             (prev->type != CT_BRACE_CLOSE) &&
-             (prev->type != CT_BRACE_OPEN) &&
-             (prev->type != CT_SEMICOLON) &&
-             (prev->parent_type != CT_TEMPLATE))
-         {
-            newline_iarf(prev, a);
+            if ((prev != NULL) && (prev->type == CT_DESTRUCTOR))
+            {
+               prev = chunk_get_prev_ncnl(prev);
+            }
+
+            /* If we are on a '::', step back two tokens
+             * TODO: do we also need to check for '.' ?
+             */
+            while ((prev != NULL) && (prev->type == CT_DC_MEMBER))
+            {
+               prev = chunk_get_prev_ncnl(prev);
+               prev = skip_template_prev(prev);
+               prev = chunk_get_prev_ncnl(prev);
+            }
+
+            if ((prev != NULL) &&
+                (prev->type != CT_BRACE_CLOSE) &&
+                (prev->type != CT_VBRACE_CLOSE) &&
+                (prev->type != CT_BRACE_OPEN) &&
+                (prev->type != CT_SEMICOLON) &&
+                (prev->parent_type != CT_TEMPLATE))
+            {
+               newline_iarf(prev, a);
+            }
          }
       }
    }
@@ -1639,7 +1691,8 @@ static void newline_func_def(chunk_t *start)
          {
             pc = tmp;
          }
-         newline_iarf(pc, cpd.settings[(start->parent_type == CT_FUNC_DEF) || (start->parent_type == CT_FUNC_CLASS) ?
+         newline_iarf(pc, cpd.settings[((start->parent_type == CT_FUNC_DEF) ||
+                                        (start->parent_type == CT_FUNC_CLASS_DEF)) ?
                                        UO_nl_func_def_args :
                                        UO_nl_func_decl_args].a);
       }
@@ -1685,6 +1738,7 @@ static void newline_func_def(chunk_t *start)
  */
 static void newline_oc_msg(chunk_t *start)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *sq_c;
    bool    one_liner = true;
@@ -1737,6 +1791,7 @@ static void newline_oc_msg(chunk_t *start)
  */
 static bool one_liner_nl_ok(chunk_t *pc)
 {
+   LOG_FUNC_ENTRY();
    LOG_FMT(LNL1LINE, "%s: check [%s] parent=[%s] flg=%" PRIx64 ", on line %d, col %d - ",
            __func__, get_token_name(pc->type), get_token_name(pc->parent_type),
            pc->flags, pc->orig_line, pc->orig_col);
@@ -1802,7 +1857,7 @@ static bool one_liner_nl_ok(chunk_t *pc)
 
       if (cpd.settings[UO_nl_func_leave_one_liners].b &&
           ((pc->parent_type == CT_FUNC_DEF) ||
-           (pc->parent_type == CT_FUNC_CLASS)))
+           (pc->parent_type == CT_FUNC_CLASS_DEF)))
       {
          LOG_FMT(LNL1LINE, "false (func def)\n");
          return(false);
@@ -1815,6 +1870,13 @@ static bool one_liner_nl_ok(chunk_t *pc)
          return(false);
       }
 
+      if (cpd.settings[UO_nl_cpp_lambda_leave_one_liners].b &&
+          ((pc->parent_type == CT_CPP_LAMBDA)))
+      {
+         LOG_FMT(LNL1LINE, "false (lambda)\n");
+         return(false);
+      }
+
       if (cpd.settings[UO_nl_oc_msg_leave_one_liner].b &&
           (pc->flags & PCF_IN_OC_MSG))
       {
@@ -1841,6 +1903,7 @@ static bool one_liner_nl_ok(chunk_t *pc)
  */
 static void undo_one_liner(chunk_t *pc)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *tmp;
 
    if (pc && (pc->flags & PCF_ONE_LINER))
@@ -1879,6 +1942,7 @@ static void undo_one_liner(chunk_t *pc)
 
 static void nl_create_one_liner(chunk_t *vbrace_open)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *tmp;
    chunk_t *first;
 
@@ -1890,11 +1954,16 @@ static void nl_create_one_liner(chunk_t *vbrace_open)
       return;
    }
 
+   int nl_total = 0;
    while ((tmp != NULL) && (tmp->type != CT_VBRACE_CLOSE))
    {
       if (chunk_is_newline(tmp))
       {
-         return;
+         nl_total += tmp->nl_count;
+         if (nl_total > 1)
+         {
+            return;
+         }
       }
       tmp = chunk_get_next(tmp);
    }
@@ -1911,6 +1980,7 @@ static void nl_create_one_liner(chunk_t *vbrace_open)
  */
 void newlines_remove_newlines(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc = chunk_get_head();
    chunk_t *next;
    chunk_t *prev;
@@ -1932,7 +2002,7 @@ void newlines_remove_newlines(void)
             pc = next;
             continue;
          }
-         else if (!chunk_is_newline(prev->next))
+         else if (prev && !chunk_is_newline(prev->next))
          {
             pc = prev;
          }
@@ -1947,6 +2017,7 @@ void newlines_remove_newlines(void)
  */
 void newlines_cleanup_braces(bool first)
 {
+   LOG_FUNC_ENTRY();
    chunk_t  *pc;
    chunk_t  *next;
    chunk_t  *prev;
@@ -2039,6 +2110,16 @@ void newlines_cleanup_braces(bool first)
       }
       else if (pc->type == CT_BRACE_OPEN)
       {
+         if ((pc->parent_type == CT_DOUBLE_BRACE) &&
+             (cpd.settings[UO_nl_paren_dbrace_open].a != AV_IGNORE))
+         {
+            prev = chunk_get_prev_ncnl(pc, CNAV_PREPROC);
+            if (chunk_is_paren_close(prev))
+            {
+               newline_iarf_pair(prev, pc, cpd.settings[UO_nl_paren_dbrace_open].a);
+            }
+         }
+
          if (cpd.settings[UO_nl_brace_brace].a != AV_IGNORE)
          {
             next = chunk_get_next_nc(pc, CNAV_PREPROC);
@@ -2080,7 +2161,8 @@ void newlines_cleanup_braces(bool first)
             next = chunk_get_next_ncnl(pc);
 
             // Handle nl_after_brace_open
-            if ((pc->level == pc->brace_level) &&
+            if (((pc->parent_type == CT_CPP_LAMBDA) ||
+                 (pc->level == pc->brace_level)) &&
                 cpd.settings[UO_nl_after_brace_open].b)
             {
                if (!one_liner_nl_ok(pc))
@@ -2126,6 +2208,24 @@ void newlines_cleanup_braces(bool first)
             }
          }
 
+         if (cpd.settings[UO_nl_brace_square].a != AV_IGNORE)
+         {
+            next = chunk_get_next_nc(pc, CNAV_PREPROC);
+            if ((next != NULL) && (next->type == CT_SQUARE_CLOSE))
+            {
+               newline_iarf_pair(pc, next, cpd.settings[UO_nl_brace_square].a);
+            }
+         }
+
+         if (cpd.settings[UO_nl_brace_fparen].a != AV_IGNORE)
+         {
+            next = chunk_get_next_nc(pc, CNAV_PREPROC);
+            if ((next != NULL) && (next->type == CT_FPAREN_CLOSE))
+            {
+               newline_iarf_pair(pc, next, cpd.settings[UO_nl_brace_fparen].a);
+            }
+         }
+
          if (cpd.settings[UO_eat_blanks_before_close_brace].b)
          {
             /* Limit the newlines before the close brace to 1 */
@@ -2174,7 +2274,7 @@ void newlines_cleanup_braces(bool first)
             }
          }
          else if (cpd.settings[UO_nl_after_brace_close].b ||
-                  (pc->parent_type == CT_FUNC_CLASS) ||
+                  (pc->parent_type == CT_FUNC_CLASS_DEF) ||
                   (pc->parent_type == CT_FUNC_DEF) ||
                   (pc->parent_type == CT_OC_MSG_DECL))
          {
@@ -2182,6 +2282,8 @@ void newlines_cleanup_braces(bool first)
             if ((next != NULL) &&
                 (next->type != CT_SEMICOLON) &&
                 (next->type != CT_COMMA) &&
+                (next->type != CT_SQUARE_CLOSE) &&
+                (next->type != CT_FPAREN_CLOSE) &&
                 ((pc->flags & (PCF_IN_ARRAY_ASSIGN | PCF_IN_TYPEDEF)) == 0) &&
                 !chunk_is_newline(next) &&
                 !chunk_is_comment(next))
@@ -2339,7 +2441,8 @@ void newlines_cleanup_braces(bool first)
       {
          if (((pc->parent_type == CT_FUNC_DEF) ||
               (pc->parent_type == CT_FUNC_PROTO) ||
-              (pc->parent_type == CT_FUNC_CLASS) ||
+              (pc->parent_type == CT_FUNC_CLASS_DEF) ||
+              (pc->parent_type == CT_FUNC_CLASS_PROTO) ||
               (pc->parent_type == CT_OPERATOR))
              &&
              ((cpd.settings[UO_nl_func_decl_start].a != AV_IGNORE) ||
@@ -2463,6 +2566,7 @@ void newlines_cleanup_braces(bool first)
  */
 static void nl_handle_define(chunk_t *pc)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *nl  = pc;
    chunk_t *ref = NULL;
 
@@ -2491,6 +2595,7 @@ static void nl_handle_define(chunk_t *pc)
 
 void newline_after_multiline_comment(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *tmp;
 
@@ -2519,6 +2624,7 @@ void newline_after_multiline_comment(void)
  */
 void newlines_insert_blank_lines(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
 
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next_ncnl(pc))
@@ -2558,6 +2664,7 @@ void newlines_insert_blank_lines(void)
 
 void newlines_squeeze_ifdef(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *ppr;
    chunk_t *pnl;
@@ -2619,6 +2726,7 @@ void newlines_squeeze_ifdef(void)
 
 void newlines_eat_start_end(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
 
    /* Process newlines at the start of the file */
@@ -2704,6 +2812,7 @@ void newlines_eat_start_end(void)
  */
 void newlines_chunk_pos(c_token_t chunk_type, tokenpos_e mode)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *next;
    chunk_t *prev;
@@ -2830,36 +2939,53 @@ void newlines_chunk_pos(c_token_t chunk_type, tokenpos_e mode)
  * Searches for CT_CLASS_COLON and moves them, if needed.
  * Also breaks up the args
  */
-void newlines_class_colon_pos(void)
+void newlines_class_colon_pos(c_token_t tok)
 {
+   LOG_FUNC_ENTRY();
    chunk_t    *pc;
    chunk_t    *next;
    chunk_t    *prev;
-   tokenpos_e mode    = cpd.settings[UO_pos_class_colon].tp;
    chunk_t    *ccolon = NULL;
+   tokenpos_e tpc, pcc;
+   argval_t   anc, ncia;
+
+   if (tok == CT_CLASS_COLON)
+   {
+      tpc = cpd.settings[UO_pos_class_colon].tp;
+      anc = cpd.settings[UO_nl_class_colon].a;
+      ncia = cpd.settings[UO_nl_class_init_args].a;
+      pcc = cpd.settings[UO_pos_class_comma].tp;
+   }
+   else /* tok == CT_CONSTR_COLON */
+   {
+      tpc = cpd.settings[UO_pos_constr_colon].tp;
+      anc = cpd.settings[UO_nl_constr_colon].a;
+      ncia = cpd.settings[UO_nl_constr_init_args].a;
+      pcc = cpd.settings[UO_pos_constr_comma].tp;
+   }
 
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next_ncnl(pc))
    {
-      if ((ccolon == NULL) && (pc->type != CT_CLASS_COLON))
+      if (!ccolon && (pc->type != tok))
       {
          continue;
       }
 
-      if (pc->type == CT_CLASS_COLON)
+      if (pc->type == tok)
       {
          ccolon = pc;
          prev   = chunk_get_prev_nc(pc);
          next   = chunk_get_next_nc(pc);
 
          if (!chunk_is_newline(prev) && !chunk_is_newline(next) &&
-             ((cpd.settings[UO_nl_class_colon].a & AV_ADD) != 0))
+             ((anc & AV_ADD) != 0))
          {
             newline_add_after(pc);
             prev = chunk_get_prev_nc(pc);
             next = chunk_get_next_nc(pc);
          }
 
-         if (cpd.settings[UO_nl_class_colon].a == AV_REMOVE)
+         if (anc == AV_REMOVE)
          {
             if (chunk_is_newline(prev) &&
                 chunk_safe_to_del_nl(prev))
@@ -2877,7 +3003,7 @@ void newlines_class_colon_pos(void)
             }
          }
 
-         if (mode & TP_TRAIL)
+         if (tpc & TP_TRAIL)
          {
             if (chunk_is_newline(prev) && (prev->nl_count == 1) &&
                 chunk_safe_to_del_nl(prev))
@@ -2885,7 +3011,7 @@ void newlines_class_colon_pos(void)
                chunk_swap(pc, prev);
             }
          }
-         else if (mode & TP_LEAD)
+         else if (tpc & TP_LEAD)
          {
             if (chunk_is_newline(next) && (next->nl_count == 1) &&
                 chunk_safe_to_del_nl(next))
@@ -2904,11 +3030,11 @@ void newlines_class_colon_pos(void)
 
          if ((pc->type == CT_COMMA) && (pc->level == ccolon->level))
          {
-            if ((cpd.settings[UO_nl_class_init_args].a & AV_ADD) != 0)
+            if ((ncia & AV_ADD) != 0)
             {
-               if (cpd.settings[UO_pos_class_comma].tp & TP_TRAIL)
+               if (pcc & TP_TRAIL)
                {
-                  if (cpd.settings[UO_nl_class_init_args].a == AV_FORCE)
+                  if (ncia == AV_FORCE)
                   {
                      newline_force_after(pc);
                   }
@@ -2917,9 +3043,9 @@ void newlines_class_colon_pos(void)
                      newline_add_after(pc);
                   }
                }
-               else if (cpd.settings[UO_pos_class_comma].tp & TP_LEAD)
+               else if (pcc & TP_LEAD)
                {
-                  if (cpd.settings[UO_nl_class_init_args].a == AV_FORCE)
+                  if (ncia == AV_FORCE)
                   {
                      newline_force_before(pc);
                   }
@@ -2935,7 +3061,7 @@ void newlines_class_colon_pos(void)
                   }
                }
             }
-            else if (cpd.settings[UO_nl_class_init_args].a == AV_REMOVE)
+            else if (ncia == AV_REMOVE)
             {
                next = chunk_get_next(pc);
                if (chunk_is_newline(next) && chunk_safe_to_del_nl(next))
@@ -2952,6 +3078,7 @@ void newlines_class_colon_pos(void)
 
 static void _blank_line_set(chunk_t *pc, const char *text, uncrustify_options uo)
 {
+   LOG_FUNC_ENTRY();
    if ((cpd.settings[uo].n > 0) && (pc->nl_count != cpd.settings[uo].n))
    {
       LOG_FMT(LBLANKD, "do_blank_lines: %s set line %d\n", text + 3, pc->orig_line);
@@ -2966,6 +3093,7 @@ static void _blank_line_set(chunk_t *pc, const char *text, uncrustify_options uo
 
 static void _blank_line_max(chunk_t *pc, const char *text, uncrustify_options uo)
 {
+   LOG_FUNC_ENTRY();
    if ((cpd.settings[uo].n > 0) && (pc->nl_count > cpd.settings[uo].n))
    {
       LOG_FMT(LBLANKD, "do_blank_lines: %s max line %d\n", text + 3, pc->orig_line);
@@ -2987,6 +3115,7 @@ static void _blank_line_max(chunk_t *pc, const char *text, uncrustify_options uo
  */
 void do_blank_lines(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *next;
    chunk_t *prev;
@@ -3106,9 +3235,8 @@ void do_blank_lines(void)
       /* Add blanks after function bodies */
       if ((prev != NULL) && (prev->type == CT_BRACE_CLOSE) &&
           ((prev->parent_type == CT_FUNC_DEF) ||
-           (prev->parent_type == CT_FUNC_CLASS) ||
+           (prev->parent_type == CT_FUNC_CLASS_DEF) ||
            (prev->parent_type == CT_OC_MSG_DECL) ||
-           (prev->parent_type == CT_OC_BLOCK_EXPR) ||
            (prev->parent_type == CT_ASSIGN)))
       {
          if (prev->flags & PCF_ONE_LINER)
@@ -3185,7 +3313,8 @@ void do_blank_lines(void)
       if ((cpd.settings[UO_nl_comment_func_def].n != 0) &&
           (pcmt->type == CT_COMMENT_MULTI) &&
           (pcmt->parent_type == CT_COMMENT_WHOLE) &&
-          (next != NULL) && ((next->parent_type == CT_FUNC_DEF) || (next->parent_type == CT_FUNC_CLASS)))
+          (next != NULL) && ((next->parent_type == CT_FUNC_DEF) ||
+                             (next->parent_type == CT_FUNC_CLASS_DEF)))
       {
          if (cpd.settings[UO_nl_comment_func_def].n != pc->nl_count)
          {
@@ -3253,6 +3382,7 @@ void do_blank_lines(void)
 
 void newlines_cleanup_dup(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *next;
 
@@ -3280,6 +3410,7 @@ void newlines_cleanup_dup(void)
  */
 static void newlines_double_space_struct_enum_union(chunk_t *open_brace)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc = open_brace;
    chunk_t *prev;
 
@@ -3312,9 +3443,9 @@ static void newlines_double_space_struct_enum_union(chunk_t *open_brace)
 
 void annotations_newlines(void)
 {
+   LOG_FUNC_ENTRY();
    chunk_t *pc;
    chunk_t *next;
-   chunk_t *prev;
    chunk_t *ae;   /* last token of the annotation */
 
    pc = chunk_get_head();
@@ -3336,31 +3467,20 @@ void annotations_newlines(void)
          break;
       }
 
-      prev = chunk_get_prev(pc);
-
       LOG_FMT(LANNOT, "%s: %d:%d annotation '%s' end@%d:%d '%s'",
               __func__, pc->orig_line, pc->orig_col, pc->text(),
               ae->orig_line, ae->orig_col, ae->text());
 
-      if (chunk_is_token(prev, CT_ANNOTATION) ||
-          !chunk_is_newline(prev))
+      next = chunk_get_next_nnl(ae);
+      if (chunk_is_token(next, CT_ANNOTATION))
       {
-         /* skip it */
-         LOG_FMT(LANNOT, " -- ignored\n");
+         LOG_FMT(LANNOT, " -- nl_between_annotation\n");
+         newline_iarf(ae, cpd.settings[UO_nl_between_annotation].a);
       }
       else
       {
-         next = chunk_get_next_nnl(ae);
-         if (chunk_is_token(next, CT_ANNOTATION))
-         {
-            LOG_FMT(LANNOT, " -- nl_between_annotation\n");
-            newline_iarf(ae, cpd.settings[UO_nl_between_annotation].a);
-         }
-         else
-         {
-            LOG_FMT(LANNOT, " -- nl_after_annotation\n");
-            newline_iarf(ae, cpd.settings[UO_nl_after_annotation].a);
-         }
+         LOG_FMT(LANNOT, " -- nl_after_annotation\n");
+         newline_iarf(ae, cpd.settings[UO_nl_after_annotation].a);
       }
    }
 }
