@@ -100,6 +100,8 @@ enum uncrustify_options
    UO_utf8_force,
    UO_utf8_bom,
 
+   UO_dont_protect_xcode_code_placeholders,
+
    UO_input_tab_size,           // tab size on input file: usually 8
    UO_output_tab_size,          // tab size for output: usually 8
 
@@ -136,6 +138,7 @@ enum uncrustify_options
    UO_indent_braces_no_class,    // whether to not indent the class braces (depends on UO_indent_braces)
    UO_indent_braces_no_struct,   // whether to not indent the struct braces (depends on UO_indent_braces)
    UO_indent_brace_parent,       // indent the braces based on the parent size (if=3, for=4, etc)
+   UO_indent_paren_open_brace,   // indent on paren level in '({', default by {
    UO_indent_label,              // 0=left >0=col from left, <0=sub from brace indent
    UO_indent_access_spec,        // same as indent_label, but for "private:", "public:"
    UO_indent_access_spec_body,   // indent private/public/protected inside a class (overrides indent_access_spec)
@@ -158,11 +161,13 @@ enum uncrustify_options
    UO_indent_func_throw,                    // indentation for standalone "throw" qualifier
 
    UO_indent_namespace,                     // indent stuff inside namespace braces
+   UO_indent_namespace_single_indent,       // indent one namespace and no sub-namepaces
    UO_indent_namespace_level,               // level to indent namespace blocks
    UO_indent_namespace_limit,               // no indent if namespace is longer than this
    UO_indent_extern,
    UO_indent_class,                         // indent stuff inside class braces
    UO_indent_class_colon,                   // indent stuff after a class colon
+   UO_indent_constr_colon,                  // indent stuff after a constr colon
 
    UO_indent_ctor_init_leading,             // virtual indent from the ':' for member initializers. Default is 2. (applies to the leading colon case)
    UO_indent_ctor_init,                     // additional indenting for ctor initializer lists
@@ -175,7 +180,13 @@ enum uncrustify_options
    UO_indent_align_assign,
    UO_indent_oc_block,
    UO_indent_oc_block_msg,
+   UO_indent_oc_msg_prioritize_first_colon,
    UO_indent_oc_msg_colon,
+   UO_indent_oc_block_msg_xcode_style,
+   UO_indent_oc_block_msg_from_brace,
+   UO_indent_oc_block_msg_from_caret,
+   UO_indent_oc_block_msg_from_colon,
+   UO_indent_oc_block_msg_from_keyword,
 
    UO_indent_else_if,
    UO_indent_var_def_blk,        // indent a variable def block that appears at the top
@@ -187,6 +198,7 @@ enum uncrustify_options
 
    UO_sp_paren_brace,           // space between ')' and '{'
    UO_sp_fparen_brace,          // space between ')' and '{' of function
+   UO_sp_fparen_dbrace,         // space between ')' and '{{' of double-brace init
    UO_sp_sparen_brace,          // space between ')' and '{' of if, while, etc
 
    UO_sp_after_cast,            // space after C & D cast - "(int) a" vs "(int)a"
@@ -229,6 +241,7 @@ enum uncrustify_options
    UO_sp_before_squares,        // space before '[]', as in 'byte []'
 
    UO_sp_paren_paren,           // space between nested parens - '( (' vs '(('
+   UO_sp_cparen_oparen,         // space between nested parens - ') (' vs ')('
    UO_sp_balance_nested_parens, // balance spaces inside nested parens
 
    UO_sp_return_paren,          // space between 'return' and '('
@@ -252,11 +265,14 @@ enum uncrustify_options
    UO_sp_assign_default,        // space around '=' in prototype
    UO_sp_before_assign,         // space before =, +=, etc
    UO_sp_after_assign,          // space after =, +=, etc
+   UO_sp_enum_paren,
    UO_sp_enum_assign,           // space around = in enum
    UO_sp_enum_before_assign,    // space before = in enum
    UO_sp_enum_after_assign,     // space after = in enum
    UO_sp_after_class_colon,     // space after class ':'
    UO_sp_before_class_colon,    // space before class ':'
+   UO_sp_after_constr_colon,
+   UO_sp_before_constr_colon,
    UO_sp_before_case_colon,
 
    UO_sp_func_def_paren,        // space between 'func' and '(' - "foo (" vs "foo("
@@ -279,6 +295,7 @@ enum uncrustify_options
    UO_sp_before_ptr_star,       // space before a '*' that is part of a type
    UO_sp_before_unnamed_ptr_star,
    UO_sp_after_ptr_star,        // space after a '*' that is part of a type
+   UO_sp_after_ptr_star_qualifier,  // space after a '*' next to a qualifier
    UO_sp_after_ptr_star_func,   // space between a '*' and a function proto/def
    UO_sp_ptr_star_paren,
    UO_sp_before_ptr_star_func,
@@ -318,6 +335,9 @@ enum uncrustify_options
    UO_sp_try_brace,
    UO_sp_getset_brace,
 
+   UO_sp_word_brace,
+   UO_sp_word_brace_ns,
+
    UO_sp_before_dc,
    UO_sp_after_dc,
    UO_sp_d_array_colon,
@@ -346,7 +366,12 @@ enum uncrustify_options
    UO_sp_after_oc_msg_receiver,
    UO_sp_after_oc_property,
    UO_sp_cond_colon,
+   UO_sp_cond_colon_before,
+   UO_sp_cond_colon_after,
    UO_sp_cond_question,
+   UO_sp_cond_question_before,
+   UO_sp_cond_question_after,
+   UO_sp_cond_ternary_short,
    UO_sp_case_label,
    UO_sp_range,
    UO_sp_cmt_cpp_start,
@@ -434,13 +459,14 @@ enum uncrustify_options
    UO_align_oc_msg_colon_span,
    UO_align_oc_msg_colon_first,
    UO_align_oc_decl_colon,
-
+   UO_align_keep_extra_space,      // don't squash extra whitespace
 
    /*
     * Newline adding and removing options
     */
 
    UO_nl_fdef_brace,                 // "int foo() {" vs "int foo()\n{"
+   UO_nl_cpp_ldef_brace,             // "[&x](int a) {" vs "[&x](int a)\n{"
    UO_nl_func_paren,                 // newline between function and open paren
    UO_nl_func_def_paren,
    UO_nl_func_decl_start,            // newline after the '(' in a function decl
@@ -475,6 +501,7 @@ enum uncrustify_options
    UO_nl_between_annotation,
    UO_nl_after_case,                 /* disallow nested "case 1: a=3;" */
    UO_nl_after_semicolon,            // disallow multiple statements on a line "a=1;b=4;"
+   UO_nl_paren_dbrace_open,
    UO_nl_after_brace_open,           // force a newline after a brace open
    UO_nl_after_brace_open_cmt,       // put the newline before the comment
    UO_nl_after_vbrace_open,          // force a newline after a virtual brace open
@@ -505,6 +532,8 @@ enum uncrustify_options
    UO_nl_getset_brace,               /* nl between get/set and { */
    UO_nl_catch_brace,                /* nl between catch and { */
    UO_nl_brace_catch,                /* nl between } and catch */
+   UO_nl_brace_square,               /* nl between } and ] */
+   UO_nl_brace_fparen,               /* nl between } and ) of a function invocation */
    UO_nl_while_brace,                /* nl between while and { */
    UO_nl_unittest_brace,             /* nl between unittest and { */
    UO_nl_scope_brace,
@@ -539,13 +568,16 @@ enum uncrustify_options
    UO_nl_end_of_file_min,            // min number of newlines at the end of the file
 
    UO_nl_class_colon,                // nl before/after class colon (tied to UO_pos_class_colon)
-   UO_nl_class_init_args,            // newline after comma in class init args
+   UO_nl_constr_colon,               // nl before/after class constr colon (tied to UO_pos_constr_colon)
+   UO_nl_class_init_args,            // newline after comma in base class list
+   UO_nl_constr_init_args,           // newline after comma in class init args
    UO_nl_collapse_empty_body,        // change { \n } into {}
    UO_nl_class_leave_one_liners,     // leave one-line function bodies in "class xx { here }"
    UO_nl_assign_leave_one_liners,    // leave one-line assign bodies in "foo_t f = { a, b, c };"
    UO_nl_enum_leave_one_liners,      // leave one-line enum bodies in "enum FOO { BAR = 5 };"
    UO_nl_getset_leave_one_liners,    // leave one-line get/set bodies
    UO_nl_func_leave_one_liners,      // leave one-line function def bodies
+   UO_nl_cpp_lambda_leave_one_liners,// leave one-line C++11 lambda bodies
    UO_nl_if_leave_one_liners,
    UO_nl_case_colon_brace,
 
@@ -564,8 +596,10 @@ enum uncrustify_options
    UO_pos_compare,                // position of trailing/leading <=/>, etc
    UO_pos_conditional,            // position of trailing/leading (b ? t : f)
    UO_pos_comma,                  // position of comma in functions
-   UO_pos_class_comma,            // position of comma in constructor init list
+   UO_pos_class_comma,            // position of comma in class parent list list
+   UO_pos_constr_comma,           // position of comma in constructor init list
    UO_pos_class_colon,            // position of trailing/leading class colon
+   UO_pos_constr_colon,           // position of trailing/leading class constr colon
 
 
    /*
@@ -644,6 +678,7 @@ enum uncrustify_options
    UO_cmt_cpp_nl_start,         // put a blank /* at the start of a converted group
    UO_cmt_cpp_nl_end,           // put a nl before the */ in a converted group
    UO_cmt_multi_check_last,     // no space after '*' prefix when comment start and end are of equal length
+   UO_cmt_convert_tab_to_spaces,
 
    UO_cmt_insert_file_header,
    UO_cmt_insert_file_footer,
@@ -655,6 +690,9 @@ enum uncrustify_options
    UO_string_escape_char,       // the string escape char to use
    UO_string_escape_char2,      // the string escape char to use
 
+   /* Hack, add comments to the ends of namespaces */
+   UO_mod_add_long_namespace_closebrace_comment,
+
    /* This is used to get the enumeration count */
    UO_option_count
 };
