@@ -35,7 +35,7 @@ static chunk_t *handle_double_angle_close(chunk_t *pc)
           (next->parent_type == CT_NONE))
       {
          pc->str.append('>');
-         pc->type = CT_ARITH;
+         set_chunk_type(pc, CT_ARITH);
          pc->orig_col_end = next->orig_col_end;
 
          chunk_t *tmp = chunk_get_next_ncnl(next);
@@ -44,7 +44,7 @@ static chunk_t *handle_double_angle_close(chunk_t *pc)
       }
       else
       {
-         pc->type = CT_COMPARE;
+         set_chunk_type(pc, CT_COMPARE);
       }
    }
    return(next);
@@ -67,7 +67,7 @@ static void split_off_angle_close(chunk_t *pc)
 
    pc->str.resize(1);
    pc->orig_col_end = pc->orig_col + 1;
-   pc->type = CT_ANGLE_CLOSE;
+   set_chunk_type(pc, CT_ANGLE_CLOSE);
 
    nc.type = ct->type;
    nc.str.pop_front();
@@ -97,7 +97,7 @@ void tokenize_cleanup(void)
          if (chunk_is_token(next, CT_SQUARE_CLOSE))
          {
             /* Change '[' + ']' into '[]' */
-            pc->type = CT_TSQUARE;
+            set_chunk_type(pc, CT_TSQUARE);
             pc->str  = "[]";
             chunk_del(next);
             pc->orig_col_end += 1;
@@ -119,7 +119,7 @@ void tokenize_cleanup(void)
    {
       if ((pc->type == CT_DOT) && ((cpd.lang_flags & LANG_ALLC) != 0))
       {
-         pc->type = CT_MEMBER;
+         set_chunk_type(pc, CT_MEMBER);
       }
 
       /* Determine the version stuff (D only) */
@@ -127,7 +127,7 @@ void tokenize_cleanup(void)
       {
          if (next->type == CT_PAREN_OPEN)
          {
-            pc->type = CT_D_VERSION_IF;
+            set_chunk_type(pc, CT_D_VERSION_IF);
          }
          else
          {
@@ -137,7 +137,7 @@ void tokenize_cleanup(void)
                        cpd.filename, pc->orig_line, __func__, get_token_name(next->type));
                cpd.error_count++;
             }
-            pc->type = CT_WORD;
+            set_chunk_type(pc, CT_WORD);
          }
       }
 
@@ -146,11 +146,11 @@ void tokenize_cleanup(void)
       {
          if (next->type == CT_PAREN_OPEN)
          {
-            pc->type = CT_D_SCOPE_IF;
+            set_chunk_type(pc, CT_D_SCOPE_IF);
          }
          else
          {
-            pc->type = CT_TYPE;
+            set_chunk_type(pc, CT_TYPE);
          }
       }
 
@@ -161,7 +161,12 @@ void tokenize_cleanup(void)
        */
       if ((pc->type == CT_BASE) && (next->type == CT_PAREN_OPEN))
       {
-         pc->type = CT_WORD;
+         set_chunk_type(pc, CT_WORD);
+      }
+
+      if ((pc->type == CT_ENUM) && (next->type == CT_CLASS))
+      {
+         set_chunk_type(next, CT_ENUM_CLASS);
       }
 
       /**
@@ -171,14 +176,15 @@ void tokenize_cleanup(void)
       if (next->type == CT_WORD)
       {
          if ((pc->type == CT_ENUM) ||
+             (pc->type == CT_ENUM_CLASS) ||
              (pc->type == CT_UNION) ||
              (pc->type == CT_STRUCT))
          {
-            next->type = CT_TYPE;
+            set_chunk_type(next, CT_TYPE);
          }
          if (pc->type == CT_WORD)
          {
-            pc->type = CT_TYPE;
+            set_chunk_type(pc, CT_TYPE);
          }
       }
 
@@ -199,9 +205,9 @@ void tokenize_cleanup(void)
          {
             /* Something else followed by a open brace */
             tmp = chunk_get_next_ncnl(next);
-            if ((tmp != NULL) || (tmp->type != CT_BRACE_OPEN))
+            if ((tmp == NULL) || (tmp->type != CT_BRACE_OPEN))
             {
-               pc->type = CT_QUALIFIER;
+               set_chunk_type(pc, CT_QUALIFIER);
             }
          }
       }
@@ -215,14 +221,14 @@ void tokenize_cleanup(void)
            (pc->type == CT_QUALIFIER) ||
            (pc->type == CT_PTR_TYPE)))
       {
-         next->type = CT_PTR_TYPE;
+         set_chunk_type(next, CT_PTR_TYPE);
       }
 
       if ((pc->type == CT_TYPE_CAST) &&
           (next->type == CT_ANGLE_OPEN))
       {
-         next->parent_type = CT_TYPE_CAST;
-         in_type_cast      = true;
+         set_chunk_parent(next, CT_TYPE_CAST);
+         in_type_cast = true;
       }
 
       /**
@@ -230,14 +236,25 @@ void tokenize_cleanup(void)
        */
       if ((pc->type == CT_ANGLE_OPEN) && (pc->parent_type != CT_TYPE_CAST))
       {
-         check_template(pc);
+         /* pretty much all languages except C use <> for something other than
+          * comparisons.  "#include<xxx>" is handled elsewhere.
+          */
+         if (cpd.lang_flags & (LANG_CPP | LANG_CS | LANG_JAVA | LANG_VALA | LANG_OC))
+         {
+            check_template(pc);
+         }
+         else
+         {
+            /* convert CT_ANGLE_OPEN to CT_COMPARE */
+            set_chunk_type(pc, CT_COMPARE);
+         }
       }
       if ((pc->type == CT_ANGLE_CLOSE) && (pc->parent_type != CT_TEMPLATE))
       {
          if (in_type_cast)
          {
-            in_type_cast    = false;
-            pc->parent_type = CT_TYPE_CAST;
+            in_type_cast = false;
+            set_chunk_parent(pc, CT_TYPE_CAST);
          }
          else
          {
@@ -253,7 +270,7 @@ void tokenize_cleanup(void)
               (prev->type == CT_WORD) ||
               (next->type == CT_STRING)))
          {
-            pc->type = CT_CONCAT;
+            set_chunk_type(pc, CT_CONCAT);
          }
 
          /* Check for the D template symbol '!' (word + '!' + word or '(') */
@@ -263,13 +280,26 @@ void tokenize_cleanup(void)
               (next->type == CT_WORD) ||
               (next->type == CT_TYPE)))
          {
-            pc->type = CT_D_TEMPLATE;
+            set_chunk_type(pc, CT_D_TEMPLATE);
          }
 
          /* handle "version(unittest) { }" vs "unittest { }" */
          if (prev && (pc->type == CT_UNITTEST) && (prev->type == CT_PAREN_OPEN))
          {
-            pc->type = CT_WORD;
+            set_chunk_type(pc, CT_WORD);
+         }
+
+         /* handle 'static if' and merge the tokens */
+         if (prev && (pc->type == CT_IF) && chunk_is_str(prev, "static", 6))
+         {
+            /* delete PREV and merge with IF */
+            pc->str.insert(0, ' ');
+            pc->str.insert(0, prev->str);
+            pc->orig_col = prev->orig_col;
+            pc->orig_line = prev->orig_line;
+            chunk_t *tmp = prev;
+            prev = chunk_get_prev_ncnl(prev);
+            chunk_del(tmp);
          }
       }
 
@@ -278,7 +308,7 @@ void tokenize_cleanup(void)
          /* Change Word before '::' into a type */
          if ((pc->type == CT_WORD) && (next->type == CT_DC_MEMBER))
          {
-            pc->type = CT_TYPE;
+            set_chunk_type(pc, CT_TYPE);
          }
       }
 
@@ -290,12 +320,12 @@ void tokenize_cleanup(void)
               (prev->type == CT_BRACE_OPEN) ||
               (prev->type == CT_SEMICOLON)))
          {
-            pc->type          = CT_GETSET_EMPTY;
-            next->parent_type = CT_GETSET;
+            set_chunk_type(pc, CT_GETSET_EMPTY);
+            set_chunk_parent(next, CT_GETSET);
          }
          else
          {
-            pc->type = CT_WORD;
+            set_chunk_type(pc, CT_WORD);
          }
       }
 
@@ -304,7 +334,7 @@ void tokenize_cleanup(void)
        */
       if ((pc->type == CT_CLASS) && !CharTable::IsKw1(next->str[0]))
       {
-         pc->type = CT_WORD;
+         set_chunk_type(pc, CT_WORD);
       }
 
       /* Change item after operator (>=, ==, etc) to a CT_OPERATOR_VAL
@@ -333,7 +363,7 @@ void tokenize_cleanup(void)
             if ((tmp != NULL) && (tmp->type == CT_PAREN_CLOSE))
             {
                next->str  = "()";
-               next->type = CT_OPERATOR_VAL;
+               set_chunk_type(next, CT_OPERATOR_VAL);
                chunk_del(tmp);
                next->orig_col_end += 1;
             }
@@ -344,16 +374,16 @@ void tokenize_cleanup(void)
          {
             next->str.append('>');
             next->orig_col_end++;
-            next->type = CT_OPERATOR_VAL;
+            set_chunk_type(next, CT_OPERATOR_VAL);
             chunk_del(tmp2);
          }
          else if (next->flags & PCF_PUNCTUATOR)
          {
-            next->type = CT_OPERATOR_VAL;
+            set_chunk_type(next, CT_OPERATOR_VAL);
          }
          else
          {
-            next->type = CT_TYPE;
+            set_chunk_type(next, CT_TYPE);
 
             /* Replace next with a collection of all tokens that are part of
              * the type.
@@ -386,11 +416,11 @@ void tokenize_cleanup(void)
                chunk_del(tmp2);
             }
 
-            next->type = CT_OPERATOR_VAL;
+            set_chunk_type(next, CT_OPERATOR_VAL);
 
             next->orig_col_end = next->orig_col + next->len();
          }
-         next->parent_type = CT_OPERATOR;
+         set_chunk_parent(next, CT_OPERATOR);
 
          LOG_FMT(LOPERATOR, "%s: %d:%d operator '%s'\n",
                  __func__, pc->orig_line, pc->orig_col, next->str.c_str());
@@ -410,7 +440,7 @@ void tokenize_cleanup(void)
          }
          if (next->type == CT_COLON)
          {
-            next->type = CT_PRIVATE_COLON;
+            set_chunk_type(next, CT_PRIVATE_COLON);
             if ((tmp = chunk_get_next_ncnl(next)) != NULL)
             {
                tmp->flags |= PCF_STMT_START | PCF_EXPR_START;
@@ -418,7 +448,7 @@ void tokenize_cleanup(void)
          }
          else
          {
-            pc->type = (chunk_is_str(pc, "signals", 7) || chunk_is_str(pc, "Q_SIGNALS", 9)) ? CT_WORD : CT_QUALIFIER;
+            set_chunk_type(pc, (chunk_is_str(pc, "signals", 7) || chunk_is_str(pc, "Q_SIGNALS", 9)) ? CT_WORD : CT_QUALIFIER);
          }
       }
 
@@ -431,15 +461,15 @@ void tokenize_cleanup(void)
             tmp = chunk_get_next(next);
             if (chunk_is_str_case(tmp, "BEGIN", 5))
             {
-               pc->type = CT_SQL_BEGIN;
+               set_chunk_type(pc, CT_SQL_BEGIN);
             }
             else if (chunk_is_str_case(tmp, "END", 3))
             {
-               pc->type = CT_SQL_END;
+               set_chunk_type(pc, CT_SQL_END);
             }
             else
             {
-               pc->type = CT_SQL_EXEC;
+               set_chunk_type(pc, CT_SQL_EXEC);
             }
 
             /* Change words into CT_SQL_WORD until CT_SEMICOLON */
@@ -451,7 +481,7 @@ void tokenize_cleanup(void)
                }
                if ((tmp->len() > 0) && unc_isalpha(*tmp->str))
                {
-                  tmp->type = CT_SQL_WORD;
+                  set_chunk_type(tmp, CT_SQL_WORD);
                }
                tmp = chunk_get_next_ncnl(tmp);
             }
@@ -476,7 +506,7 @@ void tokenize_cleanup(void)
             {
                if (chunk_is_str(tmp, "in", 2))
                {
-                  tmp->type = CT_IN;
+                  set_chunk_type(tmp, CT_IN);
                   break;
                }
                tmp = chunk_get_next_ncnl(tmp);
@@ -494,13 +524,13 @@ void tokenize_cleanup(void)
               (pc->type == CT_WHILE)) &&
              !chunk_is_token(next, CT_PAREN_OPEN))
          {
-            pc->type = CT_WORD;
+            set_chunk_type(pc, CT_WORD);
          }
          if ((pc->type == CT_DO) &&
              (chunk_is_token(prev, CT_MINUS) ||
               chunk_is_token(next, CT_SQUARE_CLOSE)))
          {
-            pc->type = CT_WORD;
+            set_chunk_type(pc, CT_WORD);
          }
       }
 
@@ -509,7 +539,7 @@ void tokenize_cleanup(void)
           (chunk_is_token(prev, CT_DOT) ||
            chunk_is_token(next, CT_DOT)))
       {
-         pc->type = CT_WORD;
+         set_chunk_type(pc, CT_WORD);
       }
 
       /* Detect Objective C class name */
@@ -519,9 +549,9 @@ void tokenize_cleanup(void)
       {
          if (next->type != CT_PAREN_OPEN)
          {
-            next->type = CT_OC_CLASS;
+            set_chunk_type(next, CT_OC_CLASS);
          }
-         next->parent_type = pc->type;
+         set_chunk_parent(next, pc->type);
 
          tmp = chunk_get_next_ncnl(next);
          if (tmp != NULL)
@@ -532,7 +562,7 @@ void tokenize_cleanup(void)
          tmp = chunk_get_next_type(pc, CT_OC_END, pc->level);
          if (tmp != NULL)
          {
-            tmp->parent_type = pc->type;
+            set_chunk_parent(tmp, pc->type);
          }
       }
 
@@ -546,7 +576,7 @@ void tokenize_cleanup(void)
                LOG_FMT(LOBJCWORD, "@interface %d:%d change '%s' (%s) to CT_WORD\n",
                        pc->orig_line, pc->orig_col, tmp->str.c_str(),
                        get_token_name(tmp->type));
-               tmp->type = CT_WORD;
+               set_chunk_type(tmp, CT_WORD);
             }
             tmp = chunk_get_next_ncnl(tmp, CNAV_PREPROC);
          }
@@ -562,27 +592,27 @@ void tokenize_cleanup(void)
            (pc->type == CT_OC_CLASS)) &&
           (next->type == CT_PAREN_OPEN))
       {
-         next->parent_type = pc->parent_type;
+         set_chunk_parent(next, pc->parent_type);
 
          tmp = chunk_get_next(next);
          if ((tmp != NULL) && (tmp->next != NULL))
          {
             if (tmp->type == CT_PAREN_CLOSE)
             {
-               //tmp->type        = CT_OC_CLASS_EXT;
-               tmp->parent_type = pc->parent_type;
+               //set_chunk_type(tmp, CT_OC_CLASS_EXT);
+               set_chunk_parent(tmp, pc->parent_type);
             }
             else
             {
-               tmp->type        = CT_OC_CATEGORY;
-               tmp->parent_type = pc->parent_type;
+               set_chunk_type(tmp, CT_OC_CATEGORY);
+               set_chunk_parent(tmp, pc->parent_type);
             }
          }
 
          tmp = chunk_get_next_type(pc, CT_PAREN_CLOSE, pc->level);
          if (tmp != NULL)
          {
-            tmp->parent_type = pc->parent_type;
+            set_chunk_parent(tmp, pc->parent_type);
          }
       }
 
@@ -598,12 +628,12 @@ void tokenize_cleanup(void)
          }
          else
          {
-            next->parent_type = pc->type;
+            set_chunk_parent(next, pc->type);
 
             tmp = chunk_get_next_type(pc, CT_PAREN_CLOSE, pc->level);
             if (tmp != NULL)
             {
-               tmp->parent_type = pc->type;
+               set_chunk_parent(tmp, pc->type);
                tmp = chunk_get_next_ncnl(tmp);
                if (tmp != NULL)
                {
@@ -612,7 +642,7 @@ void tokenize_cleanup(void)
                   tmp = chunk_get_next_type(tmp, CT_SEMICOLON, pc->level);
                   if (tmp != NULL)
                   {
-                     tmp->parent_type = pc->type;
+                     set_chunk_parent(tmp, pc->type);
                   }
                }
             }
@@ -626,23 +656,23 @@ void tokenize_cleanup(void)
        */
       if ((pc->type == CT_OC_SEL) && (next->type == CT_PAREN_OPEN))
       {
-         next->parent_type = pc->type;
+         set_chunk_parent(next, pc->type);
 
          tmp = chunk_get_next(next);
          if (tmp != NULL)
          {
-            tmp->type        = CT_OC_SEL_NAME;
-            tmp->parent_type = pc->type;
+            set_chunk_type(tmp, CT_OC_SEL_NAME);
+            set_chunk_parent(tmp, pc->type);
 
             while ((tmp = chunk_get_next_ncnl(tmp)) != NULL)
             {
                if (tmp->type == CT_PAREN_CLOSE)
                {
-                  tmp->parent_type = CT_OC_SEL;
+                  set_chunk_parent(tmp, CT_OC_SEL);
                   break;
                }
-               tmp->type        = CT_OC_SEL_NAME;
-               tmp->parent_type = pc->type;
+               set_chunk_type(tmp, CT_OC_SEL_NAME);
+               set_chunk_parent(tmp, pc->type);
             }
          }
       }
@@ -650,7 +680,7 @@ void tokenize_cleanup(void)
       /* Handle special preprocessor junk */
       if (pc->type == CT_PREPROC)
       {
-         pc->parent_type = next->type;
+         set_chunk_parent(pc, next->type);
       }
 
       /* Detect "pragma region" and "pragma endregion" */
@@ -659,9 +689,9 @@ void tokenize_cleanup(void)
          if ((memcmp(next->str, "region", 6) == 0) ||
              (memcmp(next->str, "endregion", 9) == 0))
          {
-            pc->type = (*next->str == 'r') ? CT_PP_REGION : CT_PP_ENDREGION;
+            set_chunk_type(pc, (*next->str == 'r') ? CT_PP_REGION : CT_PP_ENDREGION);
 
-            prev->parent_type = pc->type;
+            set_chunk_parent(prev, pc->type);
          }
       }
 
@@ -691,6 +721,7 @@ void tokenize_cleanup(void)
             if (doit)
             {
                pc->str += next->str;
+               pc->orig_col_end = next->orig_col_end;
                chunk_del(next);
                next = tmp;
             }
@@ -702,17 +733,19 @@ void tokenize_cleanup(void)
           (pc->type == CT_DEFAULT) &&
           (next->type == CT_PAREN_OPEN))
       {
-         pc->type = CT_SIZEOF;
+         set_chunk_type(pc, CT_SIZEOF);
       }
 
       if ((pc->type == CT_UNSAFE) && (next->type != CT_BRACE_OPEN))
       {
-         pc->type = CT_QUALIFIER;
+         set_chunk_type(pc, CT_QUALIFIER);
       }
 
-      if ((pc->type == CT_USING) && (next->type == CT_PAREN_OPEN))
+      if (((pc->type == CT_USING) ||
+           ((pc->type == CT_TRY) && (cpd.lang_flags & LANG_JAVA))) &&
+          (next->type == CT_PAREN_OPEN))
       {
-         pc->type = CT_USING_STMT;
+         set_chunk_type(pc, CT_USING_STMT);
       }
 
       /* Add minimal support for C++0x rvalue references */
@@ -720,7 +753,7 @@ void tokenize_cleanup(void)
       {
          if (prev->type == CT_TYPE)
          {
-            pc->type = CT_BYREF;
+            set_chunk_type(pc, CT_BYREF);
          }
       }
 
@@ -730,7 +763,7 @@ void tokenize_cleanup(void)
       if ((pc->type == CT_TRY) && chunk_is_str(pc, "try", 3) &&
           (next != NULL) && (next->type == CT_COLON))
       {
-         pc->type = CT_QUALIFIER;
+         set_chunk_type(pc, CT_QUALIFIER);
       }
 
       /* TODO: determine other stuff here */
@@ -813,7 +846,7 @@ static void check_template(chunk_t *start)
           (prev->parent_type != CT_OPERATOR))
       {
          LOG_FMT(LTEMPL, " - after %s + ( - Not a template\n", get_token_name(prev->type));
-         start->type = CT_COMPARE;
+         set_chunk_type(start, CT_COMPARE);
          return;
       }
 
@@ -838,7 +871,7 @@ static void check_template(chunk_t *start)
          if (pc->type == CT_SQUARE_OPEN)
          {
             LOG_FMT(LTEMPL, " - Not a template: after a square open\n");
-            start->type = CT_COMPARE;
+            set_chunk_type(start, CT_COMPARE);
             return;
          }
       }
@@ -858,8 +891,7 @@ static void check_template(chunk_t *start)
 
          if ((tokens[num_tokens - 1] == CT_ANGLE_OPEN) &&
              (pc->str[0] == '>') && (pc->len() > 1) &&
-             (cpd.settings[UO_tok_split_gte].b ||
-              (chunk_is_str(pc, ">>", 2) && ((cpd.lang_flags & LANG_CPP) == 0))))
+             (cpd.settings[UO_tok_split_gte].b || chunk_is_str(pc, ">>", 2)))
          {
             LOG_FMT(LTEMPL, " {split '%s' at %d:%d}",
                     pc->str.c_str(), pc->orig_line, pc->orig_col);
@@ -926,7 +958,7 @@ static void check_template(chunk_t *start)
       {
          LOG_FMT(LTEMPL, " - Template Detected\n");
 
-         start->parent_type = CT_TEMPLATE;
+         set_chunk_parent(start, CT_TEMPLATE);
 
          pc = start;
          while (pc != end)
@@ -939,13 +971,13 @@ static void check_template(chunk_t *start)
             }
             pc = next;
          }
-         end->parent_type = CT_TEMPLATE;
-         end->flags      |= PCF_IN_TEMPLATE;
+         set_chunk_parent(end, CT_TEMPLATE);
+         end->flags |= PCF_IN_TEMPLATE;
          return;
       }
    }
 
    LOG_FMT(LTEMPL, " - Not a template: end = %s\n",
            (end != NULL) ? get_token_name(end->type) : "<null>");
-   start->type = CT_COMPARE;
+   set_chunk_type(start, CT_COMPARE);
 }
