@@ -88,6 +88,7 @@ struct paren_stack_entry
    c_token_t    parent;       /**< if, for, function, etc */
    brstage_e    stage;
    bool         in_preproc;   /**< whether this was created in a preprocessor */
+   int          ns_cnt;
    bool         non_vardef;   /**< Hit a non-vardef line */
    indent_ptr_t ip;
 };
@@ -129,9 +130,10 @@ struct parse_frame
 #define PCF_IN_CONST_ARGS      PCF_BIT(8)
 #define PCF_IN_ARRAY_ASSIGN    PCF_BIT(9)
 #define PCF_IN_CLASS           PCF_BIT(10)
-#define PCF_IN_NAMESPACE       PCF_BIT(11)
-#define PCF_IN_FOR             PCF_BIT(12)
-#define PCF_IN_OC_MSG          PCF_BIT(13)
+#define PCF_IN_CLASS_BASE      PCF_BIT(11)
+#define PCF_IN_NAMESPACE       PCF_BIT(12)
+#define PCF_IN_FOR             PCF_BIT(13)
+#define PCF_IN_OC_MSG          PCF_BIT(14)
 
 /* Non-Copy flags are in the upper 48 bits */
 #define PCF_FORCE_SPACE        PCF_BIT(16)  /* must have a space after this token */
@@ -174,10 +176,10 @@ static const char *pcf_names[] =
    "IN_CONST_ARGS",     // 8
    "IN_ARRAY_ASSIGN",   // 9
    "IN_CLASS",          // 10
-   "IN_NAMESPACE",      // 11
-   "IN_FOR",            // 12
-   "IN_OC_MSG",         // 13
-   "#14",               // 14
+   "IN_CLASS_BASE",     // 11
+   "IN_NAMESPACE",      // 12
+   "IN_FOR",            // 13
+   "IN_OC_MSG",         // 14
    "#15",               // 15
    "FORCE_SPACE",       // 16
    "STMT_START",        // 17
@@ -241,6 +243,7 @@ struct chunk_t
       orig_line = 0;
       orig_col = 0;
       orig_col_end = 0;
+      orig_prev_sp = 0;
       flags = 0;
       column = 0;
       column_indent = 0;
@@ -269,6 +272,7 @@ struct chunk_t
    UINT32       orig_line;
    UINT32       orig_col;
    UINT32       orig_col_end;
+   UINT32       orig_prev_sp;     /* whitespace before this token */
    UINT64       flags;            /* see PCF_xxx */
    int          column;           /* column of chunk */
    int          column_indent;    /* if 1st on a line, set to the 'indent'
@@ -350,7 +354,11 @@ struct file_mem
 
 struct cp_data
 {
+   deque<UINT8>       *bout;
    FILE               *fout;
+   int                last_char;
+   bool               do_check;
+   int                check_fail_cnt; // total failures
 
    UINT32             error_count;
    const char         *filename;
@@ -369,6 +377,8 @@ struct cp_data
    UINT16             column;  /* column for parsing */
    UINT16             spaces;  /* space count on output */
 
+   int                ifdef_over_whole_file;
+
    bool               frag;
    UINT16             frag_cols;
 
@@ -381,6 +391,8 @@ struct cp_data
    int                did_newline;
    c_token_t          in_preproc;
    int                preproc_ncnl_count;
+   bool               output_trailspace;
+   bool               output_tab_as_space;
 
    bool               bom;
    CharEncoding       enc;
