@@ -6,20 +6,20 @@ class CFooBarDlg : public CDialog
 // Construction
  public:
    CFooBarDlg(CFooBar *pDataMan,
-              CWnd    *pParent = NULL);
+              CWnd    *pParent=NULL);
    virtual ~CFooBarDlg();
 
-   void Initialize(BYTE nDelay);
+   void Initialize(BYTE nDelay=100);
 
    UINT GetCount() { return(m_nCount); }
 
-   void SetCount(int count)
+   void SetCount(int count=1)
    {
       if ((count > 0) && (count < MAX_COUNT))
       {
          m_nCount = count;
       }
-   };
+   }
 
    // Dialog Data
    //{{AFX_DATA(CATCSMgrDlg)
@@ -47,6 +47,5 @@ class CFooBarDlg : public CDialog
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()
 };
-
 #endif   /* _FOO_BAR_H_INCLUDED_ */
 
